(function (deckyFrontendLib, React) {
    'use strict';

    function _interopDefaultLegacy (e) { return e && typeof e === 'object' && 'default' in e ? e : { 'default': e }; }

    var React__default = /*#__PURE__*/_interopDefaultLegacy(React);

    // import { name } from '../plugin.json'
    const pluginName = 'Web Browser';
    const routePath = '/webbrowser';
    const defaultUrl = 'https://store.steampowered.com';
    const windowRouter = deckyFrontendLib.Router.WindowStore?.GamepadUIMainWindowInstance;
    const SP_Window = windowRouter?.BrowserWindow;
    const reactTree = deckyFrontendLib.getReactRoot(document.getElementById('root'));

    const logN = (name, ...args) => {
        console.log(`%c ${pluginName} %c ${name} %c`, 'background: #7627D3; color: #E5E1EA;', 'background: #AF70F9; color: black;', 'background: transparent;', ...args);
    };
    const warnN = (name, ...args) => {
        console.warn(`%c ${pluginName} %c ${name} %c`, 'background: #7627D3; color: #E5E1EA;', 'background: #AF70F9; color: black;', 'background: transparent;', ...args);
    };
    const errorN = (name, ...args) => {
        console.error(`%c ${pluginName} %c ${name} %c`, 'background: #7627D3; color: #E5E1EA;', 'background: #AF70F9; color: black;', 'background: transparent;', ...args);
    };

    class BackendService {
        constructor() {
            this.serverApi = null;
        }
        init(serverApi) {
            this.serverApi = serverApi;
        }
        runInTarget(id, fn, async = false, ...args) {
            let strArgs;
            if (args) {
                strArgs = args.map((arg) => {
                    return JSON.stringify(arg);
                });
            }
            const code = '(' + fn.toString() + ')' + `(${strArgs ? strArgs.join(',') : ''})`;
            return this.serverApi.callPluginMethod('execute_in_target', { code, frontendId: id, run_async: async });
        }
    }
    const backendService = new BackendService();

    const status = {
        running: false,
        noTabBar: false
    };

    var SearchEngine;
    (function (SearchEngine) {
        SearchEngine[SearchEngine["GOOGLE"] = 0] = "GOOGLE";
        SearchEngine[SearchEngine["BING"] = 1] = "BING";
        SearchEngine[SearchEngine["YAHOO"] = 2] = "YAHOO";
        SearchEngine[SearchEngine["CUSTOM"] = 3] = "CUSTOM";
    })(SearchEngine || (SearchEngine = {}));
    class SettingsManager {
        constructor() {
            this.settingsLoaded = false;
            this.favoritesLoaded = false;
            this.historyLoaded = false;
            this.settings = { defaultTabs: ['home'], menuPosition: 3 };
            this.favorites = {};
            this.history = {};
        }
        init() {
            this.load();
        }
        load() {
            if (!backendService.serverApi)
                throw new Error('The Server API has not been initialized');
            this.settingsLoad = backendService.serverApi.callPluginMethod('load', { manager: 'settings' });
            this.favoritesLoad = backendService.serverApi.callPluginMethod('load', { manager: 'favorites' });
            this.historyLoad = backendService.serverApi.callPluginMethod('load', { manager: 'history' });
            this.settingsLoad.then(({ success, result }) => {
                if (success) {
                    if (Object.keys(result).length === 0) {
                        this.saveDataSet('settings');
                    }
                    else {
                        this.settings = result;
                        if (!result.menuPosition) {
                            this.settings.menuPosition = 3;
                            this.saveDataSet('settings');
                        }
                        status.noTabBar = !!this.settings.noTabBar;
                    }
                    this.settingsLoaded = true;
                }
                else
                    errorN('Settings Manager', 'Failed to load settings');
            });
            this.favoritesLoad.then(({ success, result }) => {
                if (success) {
                    this.favorites = result;
                    this.favoritesLoaded = true;
                }
                else
                    errorN('Settings Manager', 'Failed to load favorites');
            });
            this.historyLoad.then(({ success, result }) => {
                if (success) {
                    this.history = result;
                    this.historyLoaded = true;
                }
                else
                    errorN('Settings Manager', 'Failed to load history');
            });
        }
        saveSetting(settingName) {
            backendService.serverApi.callPluginMethod('save_setting', { key: settingName, value: this.settings[settingName] });
        }
        setSetting(settingName, value) {
            this.settings[settingName] = value;
            this.saveSetting(settingName);
        }
        saveDataSet(dataSet) {
            backendService.serverApi.callPluginMethod('save_data', { manager: dataSet, data: this[dataSet] });
        }
    }
    const settingsManager = new SettingsManager();

    /**
     * A component for creating reorderable lists.
     *
     * See an example implementation {@linkplain https://github.com/Tormak9970/Component-Testing-Plugin/blob/main/src/testing-window/ReorderableListTest.tsx here}.
     */
    function ReorderableList(props) {
        if (props.animate === undefined)
            props.animate = true;
        const [entryList, setEntryList] = React.useState(props.entries.sort((a, b) => a.position - b.position));
        const [reorderEnabled, setReorderEnabled] = React.useState(false);
        React.useEffect(() => {
            setEntryList(props.entries.sort((a, b) => a.position - b.position));
        }, [props.entries]);
        function toggleReorderEnabled() {
            let newReorderValue = !reorderEnabled;
            setReorderEnabled(newReorderValue);
            if (!newReorderValue) {
                props.onSave(entryList);
            }
        }
        function handleButton(e) {
            const event = e;
            const button = event.detail.button;
            if (button === (props.reorderButton ? props.reorderButton : deckyFrontendLib.GamepadButton.SECONDARY)) {
                toggleReorderEnabled();
            }
            if (reorderEnabled) {
                if (button === deckyFrontendLib.GamepadButton.CANCEL || (button === deckyFrontendLib.GamepadButton.OK && props.saveOrderOnOk))
                    setReorderEnabled(false);
                props.onSave(entryList);
            }
        }
        const actionDescription = {
            [props.reorderButton || deckyFrontendLib.GamepadButton.SECONDARY]: reorderEnabled ? 'Save Order' : 'Reorder',
        };
        if (props.saveOrderOnOk) {
            actionDescription[deckyFrontendLib.GamepadButton.OK] = reorderEnabled ? '' : 'Select';
        }
        return (window.SP_REACT.createElement(React.Fragment, null,
            window.SP_REACT.createElement("div", { style: {
                    width: 'inherit',
                    height: 'inherit',
                    flex: '1 1 1px',
                    scrollPadding: '48px 0px',
                    display: 'flex',
                    flexDirection: 'column',
                    justifyContent: 'flex-start',
                    alignContent: 'stretch',
                    padding: '0px 16px'
                } },
                window.SP_REACT.createElement(deckyFrontendLib.Focusable, { actionDescriptionMap: actionDescription, onClick: toggleReorderEnabled, onButtonDown: handleButton }, entryList.map((entry) => (window.SP_REACT.createElement(ReorderableItem, { animate: props.animate, listData: entryList, entryData: entry, reorderEntryFunc: setEntryList, reorderEnabled: reorderEnabled, fieldProps: props.fieldProps, onButtonPress: props.onButtonPress }, props.interactables ? window.SP_REACT.createElement(props.interactables, { entry: entry }) : null)))))));
    }
    function ReorderableItem(props) {
        const [isSelected, _setIsSelected] = React.useState(false);
        const [isSelectedLastFrame, setIsSelectedLastFrame] = React.useState(false);
        const listEntries = props.listData;
        function onReorder(e) {
            const event = e;
            const getIndex = () => { return listEntries.findIndex((entryData) => entryData === props.entryData); };
            if (!props.reorderEnabled) {
                props.onButtonPress && props.onButtonPress(e, props.entryData);
                return;
            }
            const currentIdx = getIndex();
            const currentIdxValue = listEntries[currentIdx];
            if (currentIdx < 0)
                return;
            let targetPosition = -1;
            if (event.detail.button == deckyFrontendLib.GamepadButton.DIR_DOWN) {
                targetPosition = currentIdxValue.position + 1;
            }
            else if (event.detail.button == deckyFrontendLib.GamepadButton.DIR_UP) {
                targetPosition = currentIdxValue.position - 1;
            }
            if (targetPosition >= listEntries.length || targetPosition < 0)
                return;
            let otherToUpdate = listEntries.find((entryData) => entryData.position === targetPosition);
            if (!otherToUpdate)
                return;
            let currentPosition = currentIdxValue.position;
            currentIdxValue.position = otherToUpdate.position;
            otherToUpdate.position = currentPosition;
            props.reorderEntryFunc([...listEntries].sort((a, b) => a.position - b.position));
        }
        async function setIsSelected(val) {
            _setIsSelected(val);
            // Wait 3 frames, then set. I have no idea why, but if you dont wait long enough it doesn't work.
            for (let i = 0; i < 3; i++)
                await new Promise((res) => requestAnimationFrame(res));
            setIsSelectedLastFrame(val);
        }
        if (!props.fieldProps)
            props.fieldProps = {};
        const { actionDescriptionMap, onOKActionDescription, onSecondaryActionDescription, onOptionsActionDescription, onMenuActionDescription, ...otherFieldProps } = props.fieldProps;
        const actionDescriptionProps = props.reorderEnabled ?
            {} :
            {
                actionDescriptionMap,
                onOKActionDescription,
                onSecondaryActionDescription,
                onOptionsActionDescription,
                onMenuActionDescription,
            };
        return (window.SP_REACT.createElement("div", { style: props.animate
                ? {
                    transition: isSelected || isSelectedLastFrame
                        ? ''
                        : 'transform 0.3s cubic-bezier(0.25, 1, 0.5, 1), opacity 0.3s cubic-bezier(0.25, 1, 0.5, 1)',
                    transform: !props.reorderEnabled || isSelected ? 'scale(1)' : 'scale(0.9)',
                    opacity: !props.reorderEnabled || isSelected ? 1 : 0.7,
                }
                : {} },
            window.SP_REACT.createElement(deckyFrontendLib.Field, { label: props.entryData.label, description: props.entryData.description, ...actionDescriptionProps, ...otherFieldProps, focusable: !props.children, onButtonDown: onReorder, onGamepadBlur: () => setIsSelected(false), onGamepadFocus: () => setIsSelected(true) },
                window.SP_REACT.createElement(deckyFrontendLib.Focusable, { style: { display: 'flex', width: '100%', position: 'relative' } }, props.children))));
    }

    class StructureController {
        constructor(dataStructure) {
            this.dataStructure = dataStructure;
        }
        clone() {
            return StructureController.clone(this.dataStructure);
        }
        addChildMember(newMemberName, value, lineageArrayToParent, isComplex) {
            StructureController.addChildMember(this.dataStructure, newMemberName, value, lineageArrayToParent, isComplex);
        }
        addPrimitiveChildMember(newMemberName, value, lineageArrayToParent) {
            StructureController.addPrimitiveChildMember(this.dataStructure, newMemberName, value, lineageArrayToParent);
        }
        addComplexChildMember(newMemberName, lineageArrayToParent) {
            StructureController.addComplexChildMember(this.dataStructure, newMemberName, lineageArrayToParent);
        }
        addComplexChildMemberWithValue(dataStructure, newMemberName, value, lineageArrayToParent) {
            StructureController.addComplexChildMemberWithValue(this.dataStructure, newMemberName, value, lineageArrayToParent);
        }
        traverseAndGetMember(lineageArray) {
            return StructureController.traverseAndGetMember(this.dataStructure, lineageArray);
        }
        traverseAndGetMemberParent(lineageArray) {
            return StructureController.traverseAndGetMemberParent(this.dataStructure, lineageArray);
        }
        removeMember(lineageArray) {
            StructureController.removeMember(this.dataStructure, lineageArray);
        }
        map(mappingFn, externalArgs, mapTopLevel, lineageArray) {
            return StructureController.map(this.dataStructure, mappingFn, externalArgs, mapTopLevel, lineageArray);
        }
        isMemberComplex(lineageArray) {
            return (typeof StructureController.isMemberComplex(this.dataStructure, lineageArray) === 'object');
        }
        doesLineageExist(lineageArray) {
            return StructureController.doesLineageExist(this.dataStructure, lineageArray);
        }
        mergeMembersWith(sourceDataStructure, complexTargetCollisonWithPrimitiveSourcePriority = 'target', primitiveTargetCollisonWithComplexSourcePriority = 'target', primitiveTargetCollisonWithPrimitiveSourcePriority = 'target', cloneSource) {
            StructureController.mergeMembers(this.dataStructure, sourceDataStructure, complexTargetCollisonWithPrimitiveSourcePriority, primitiveTargetCollisonWithComplexSourcePriority, primitiveTargetCollisonWithPrimitiveSourcePriority, cloneSource);
        }
        mergeSourceToDescendant(sourceDataStructure, targetMemberLineageArray, complexTargetCollisonWithPrimitiveSourcePriority = 'target', primitiveTargetCollisonWithComplexSourcePriority = 'target', primitiveTargetCollisonWithPrimitiveSourcePriority = 'target') {
            StructureController.mergeSourceToTargetDescendant(this.dataStructure, sourceDataStructure, targetMemberLineageArray, complexTargetCollisonWithPrimitiveSourcePriority, primitiveTargetCollisonWithComplexSourcePriority, primitiveTargetCollisonWithPrimitiveSourcePriority);
        }
        mergeWithSourceDescendant(sourceDataStructure, sourceMemberLineageArray, complexTargetCollisonWithPrimitiveSourcePriority = 'target', primitiveTargetCollisonWithComplexSourcePriority = 'target', primitiveTargetCollisonWithPrimitiveSourcePriority = 'target') {
            StructureController.mergeSourceDescendantToTarget(this.dataStructure, sourceDataStructure, sourceMemberLineageArray, complexTargetCollisonWithPrimitiveSourcePriority, primitiveTargetCollisonWithComplexSourcePriority, primitiveTargetCollisonWithPrimitiveSourcePriority);
        }
        mergeSourceDescendantToDescendant(sourceDataStructure, targetMemberLineageArray, sourceMemberLineageArray, complexTargetCollisonWithPrimitiveSourcePriority = 'target', primitiveTargetCollisonWithComplexSourcePriority = 'target', primitiveTargetCollisonWithPrimitiveSourcePriority = 'target') {
            StructureController.mergeSourceDescendantToTargetDescendant(this.dataStructure, sourceDataStructure, targetMemberLineageArray, sourceMemberLineageArray, complexTargetCollisonWithPrimitiveSourcePriority, primitiveTargetCollisonWithComplexSourcePriority, primitiveTargetCollisonWithPrimitiveSourcePriority);
        }
        static lineageArrayToString(lineageArray, separator, removeTrailingSeparator) {
            let output = '';
            for (let i = 0; i < lineageArray.length; i++) {
                output += lineageArray[i];
                if (!(i === lineageArray.length - 1 && removeTrailingSeparator)) {
                    output += separator;
                }
            }
            return output;
        }
        static addChildMember(dataStructure, newMemberName, value, lineageArrayToParent, isComplex) {
            if (!isComplex && typeof value === 'object') {
                throw new Error('Cannot assign a Complex value to a Primitive member. Value should not be an Object or Array.');
            }
            if (isComplex && typeof value !== 'object') {
                throw new Error('Cannot assign a Primitive value to a Complex member. Value should be an Object or Array.');
            }
            const parent = StructureController.traverseAndGetMember(dataStructure, lineageArrayToParent);
            if (typeof parent !== 'object') {
                throw new Error(`Cannot add member to location. Destination is a Primitive member and cannot have children.`);
            }
            parent[newMemberName] = value;
        }
        static addPrimitiveChildMember(dataStructure, newMemberName, value, lineageArrayToParent) {
            StructureController.addChildMember(dataStructure, newMemberName, value, lineageArrayToParent, false);
        }
        static addComplexChildMember(dataStructure, newMemberName, lineageArrayToParent) {
            StructureController.addChildMember(dataStructure, newMemberName, {}, lineageArrayToParent, true);
        }
        static addComplexChildMemberWithValue(dataStructure, newMemberName, value, lineageArrayToParent) {
            StructureController.addChildMember(dataStructure, newMemberName, value, lineageArrayToParent, true);
        }
        static traverseAndGetMember(dataStructure, lineageArray) {
            let parent = dataStructure;
            let traversed = ['origin'];
            for (let memberName of lineageArray) {
                if (!parent.hasOwnProperty(memberName)) {
                    throw new Error(`Cannot find member "${memberName}" at location "${StructureController.lineageArrayToString(traversed, '> ', true)}".`);
                }
                parent = parent[memberName];
                traversed.push(memberName);
            }
            return parent;
        }
        static traverseAndGetMemberParent(dataStructure, lineageArray) {
            return StructureController.traverseAndGetMember(dataStructure, lineageArray.slice(0, -1));
        }
        static removeMember(dataStructure, lineageArray) {
            if (lineageArray.length < 1) {
                throw new Error('Cannot delete top level of structure.');
            }
            const memberToDelete = lineageArray[lineageArray.length - 1];
            const parent = StructureController.traverseAndGetMemberParent(dataStructure, lineageArray);
            if (!parent.hasOwnProperty(memberToDelete)) {
                throw new Error(`Member at "${StructureController.lineageArrayToString(['origin', ...lineageArray], '> ', true)}" does not contain child member "${memberToDelete}" to delete.`);
            }
            delete parent[memberToDelete];
        }
        static map(fullStructure, mappingFn, externalArgs = [], onlyMapDescendants, lineageArray) {
            if (externalArgs && !Array.isArray(externalArgs)) {
                throw new Error('External args must be and Array in order to pass arguments to mapping function.');
            }
            const children = [];
            if (!lineageArray)
                lineageArray = [];
            const subStructure = StructureController.traverseAndGetMember(fullStructure, lineageArray);
            for (let memberName in subStructure) {
                const member = subStructure[memberName];
                let mappedChild;
                if (typeof member === 'object') {
                    mappedChild = mappingFn(StructureController.map(fullStructure, mappingFn, externalArgs, true, [...lineageArray, memberName]), member, ['origin', ...lineageArray, memberName], ...externalArgs);
                }
                else {
                    mappedChild = mappingFn(null, member, ['origin', ...lineageArray, memberName], ...externalArgs);
                }
                if (mappedChild)
                    children.push(mappedChild);
            }
            if (!onlyMapDescendants)
                return mappingFn(children, subStructure, ['origin', ...lineageArray], ...externalArgs);
            return children;
        }
        static isMemberComplex(dataStructure, lineageArray) {
            return (typeof StructureController.traverseAndGetMember(dataStructure, lineageArray) === 'object');
        }
        static doesLineageExist(dataStructure, lineageArray) {
            let parent = dataStructure;
            for (let memberName of lineageArray) {
                if (!parent.hasOwnProperty(memberName)) {
                    return false;
                }
                parent = parent[memberName];
            }
            return true;
        }
        static mergeMembers(targetDataStructure, sourceDataStructure, complexTargetCollisonWithPrimitiveSourcePriority = 'target', primitiveTargetCollisonWithComplexSourcePriority = 'target', primitiveTargetCollisonWithPrimitiveSourcePriority = 'target', cloneSource) {
            if (typeof sourceDataStructure !== 'object') {
                throw new Error('Cannot merge. Source is not a data structure');
            }
            if (typeof sourceDataStructure !== 'object') {
                throw new Error('Cannot merge. Target is not a data structure');
            }
            if (cloneSource) {
                sourceDataStructure = structuredClone(sourceDataStructure);
            }
            for (let memberName in sourceDataStructure) {
                if (targetDataStructure.hasOwnProperty(memberName)) {
                    if (typeof targetDataStructure[memberName] === 'object') {
                        if (typeof sourceDataStructure[memberName] === 'object') {
                            StructureController.mergeMembers(targetDataStructure[memberName], sourceDataStructure[memberName], complexTargetCollisonWithPrimitiveSourcePriority, primitiveTargetCollisonWithComplexSourcePriority, primitiveTargetCollisonWithPrimitiveSourcePriority);
                        }
                        else {
                            if (complexTargetCollisonWithPrimitiveSourcePriority === 'source') {
                                targetDataStructure[memberName] = sourceDataStructure[memberName];
                            }
                        }
                    }
                    else {
                        if (typeof sourceDataStructure[memberName] === 'object') {
                            if (primitiveTargetCollisonWithComplexSourcePriority === 'source') {
                                targetDataStructure[memberName] = sourceDataStructure[memberName];
                            }
                        }
                        else {
                            if (primitiveTargetCollisonWithPrimitiveSourcePriority === 'source') {
                                targetDataStructure[memberName] = sourceDataStructure[memberName];
                            }
                        }
                    }
                }
                else {
                    targetDataStructure[memberName] = sourceDataStructure[memberName];
                }
            }
        }
        static mergeSourceToTargetDescendant(targetDataStructure, sourceDataStructure, targetMemberLineageArray, complexTargetCollisonWithPrimitiveSourcePriority = 'target', primitiveTargetCollisonWithComplexSourcePriority = 'target', primitiveTargetCollisonWithPrimitiveSourcePriority = 'target') {
            const targetMember = StructureController.traverseAndGetMember(targetDataStructure, targetMemberLineageArray);
            StructureController.mergeMembers(targetMember, sourceDataStructure, complexTargetCollisonWithPrimitiveSourcePriority, primitiveTargetCollisonWithComplexSourcePriority, primitiveTargetCollisonWithPrimitiveSourcePriority, true);
        }
        static mergeSourceDescendantToTarget(targetDataStructure, sourceDataStructure, sourceMemberLineageArray, complexTargetCollisonWithPrimitiveSourcePriority = 'target', primitiveTargetCollisonWithComplexSourcePriority = 'target', primitiveTargetCollisonWithPrimitiveSourcePriority = 'target') {
            const sourceMember = StructureController.traverseAndGetMember(sourceDataStructure, sourceMemberLineageArray);
            StructureController.mergeMembers(targetDataStructure, sourceMember, complexTargetCollisonWithPrimitiveSourcePriority, primitiveTargetCollisonWithComplexSourcePriority, primitiveTargetCollisonWithPrimitiveSourcePriority, true);
        }
        static mergeSourceDescendantToTargetDescendant(targetDataStructure, sourceDataStructure, targetMemberLineageArray, sourceMemberLineageArray, complexTargetCollisonWithPrimitiveSourcePriority = 'target', primitiveTargetCollisonWithComplexSourcePriority = 'target', primitiveTargetCollisonWithPrimitiveSourcePriority = 'target') {
            const targetMember = StructureController.traverseAndGetMember(targetDataStructure, targetMemberLineageArray);
            const sourceMember = StructureController.traverseAndGetMember(sourceDataStructure, sourceMemberLineageArray);
            StructureController.mergeMembers(targetMember, sourceMember, complexTargetCollisonWithPrimitiveSourcePriority, primitiveTargetCollisonWithComplexSourcePriority, primitiveTargetCollisonWithPrimitiveSourcePriority, true);
        }
        static clone(dataStructure) {
            if (typeof dataStructure !== 'object') {
                throw new Error('No Data structure to clone');
            }
            return structuredClone(dataStructure);
        }
    }

    class FavoritesManager extends StructureController {
        constructor() {
            const data = {};
            super(data);
            this.loaded = false;
            this.favorites = data;
        }
        //make sure to init settings manager before this
        async init() {
            await settingsManager.favoritesLoad;
            this.loaded = true;
            this.favorites = settingsManager.favorites;
            this.dataStructure = this.favorites;
        }
        pathToString(pathArray, separator, removeTrailingSeparator) {
            return StructureController.lineageArrayToString(pathArray, separator, removeTrailingSeparator);
        }
        doesExist(pathArray, isFolder) {
            const lastIndex = pathArray.length - 1;
            const name = pathArray[lastIndex];
            const path = pathArray.slice(1, -1);
            path.push(isFolder ? name : name + '_P');
            return this.doesLineageExist(path);
        }
        getFavorite(pathArray) {
            const path = pathArray.slice(1);
            path[path.length - 1] = pathArray[pathArray.length - 1] + '_P';
            return this.traverseAndGetMember(path);
        }
        addFolder(folderName, parentPathArray) {
            this.addComplexChildMember(folderName, parentPathArray.slice(1));
            this.save();
        }
        addFavorite(favoriteName, url, parentPathArray) {
            this.addPrimitiveChildMember(favoriteName + '_P', url, parentPathArray.slice(1));
            this.save();
        }
        delete(pathArray) {
            this.removeMember(pathArray.slice(1));
            this.save();
        }
        save() {
            settingsManager.saveDataSet('favorites');
        }
    }
    const favoritesManager = new FavoritesManager();

    const classes = {
        tabbedBrowserContainer: 'tabbedBrowserContainer',
        destructiveModal: 'destructiveModal',
    };
    const spinnerClasses = deckyFrontendLib.findClassModule(m => !!m['SpinnerLoaderContainer']);
    const searchBarClasses = deckyFrontendLib.findClassModule(m => !!m['SearchAndTitleContainer']);
    const browserClasses = deckyFrontendLib.findClassModule(m => !!m['MainBrowserContainer']);

    const ConfirmFavoriteOverwriteModal = ({ name, oldUrl, newUrl, onConfirm, closeModal }) => {
        return (window.SP_REACT.createElement(deckyFrontendLib.ConfirmModal, { className: classes.destructiveModal, strTitle: 'Are you sure you want to overwrite ' + name, strCancelButtonText: 'Back', onOK: () => {
                closeModal();
                onConfirm();
            }, onCancel: closeModal },
            window.SP_REACT.createElement("div", { style: { marginTop: '10px', top: '10px', position: 'relative' } },
                window.SP_REACT.createElement("div", null,
                    window.SP_REACT.createElement("span", { style: { width: '150px' } }, "Old Url"),
                    window.SP_REACT.createElement(deckyFrontendLib.Marquee, { style: { left: '70px', top: '-20px', width: '88%' }, play: true }, oldUrl)),
                window.SP_REACT.createElement("div", null,
                    window.SP_REACT.createElement("span", { style: { width: '90px' } }, "New Url"),
                    window.SP_REACT.createElement(deckyFrontendLib.Marquee, { style: { left: '70px', top: '-20px', width: '88%' }, play: true }, newUrl)))));
    };
    const ConfirmFavoriteDeleteModal = ({ path, url, isFolder, closeModal }) => {
        const name = path[path.length - 1];
        const pathStr = favoritesManager.pathToString(path, '> ', true);
        const body = [];
        if (isFolder) {
            body[0] = pathStr;
        }
        else {
            body[0] = pathStr.slice(0, -2);
            body[1] = (window.SP_REACT.createElement(deckyFrontendLib.Marquee, { style: { position: 'relative', top: '-120px' }, play: true, center: true }, url));
        }
        return (window.SP_REACT.createElement(deckyFrontendLib.ConfirmModal, { className: classes.destructiveModal, strTitle: 'Are you sure you want to delete ' + (isFolder ? 'folder ' + name : name.slice(0, -2)), strOKButtonText: 'Delete', strCancelButtonText: 'Cancel', onOK: () => {
                favoritesManager.delete(path);
                closeModal();
            }, onCancel: closeModal },
            window.SP_REACT.createElement("div", { style: { height: 15 } }, ...body)));
    };
    const ConfirmDeleteDefaultTabModal = ({ index, closeModal }) => {
        return (window.SP_REACT.createElement(deckyFrontendLib.ConfirmModal, { className: classes.destructiveModal, strTitle: 'Are you sure you want to delete this default tabs', strOKButtonText: 'Delete', strCancelButtonText: 'Cancel', onOK: () => {
                settingsManager.settings.defaultTabs.splice(index, 1);
                settingsManager.saveSetting('defaultTabs');
                closeModal();
            }, onCancel: closeModal },
            window.SP_REACT.createElement("div", { style: { height: 15 } },
                window.SP_REACT.createElement(deckyFrontendLib.Marquee, { play: true }, settingsManager.settings.defaultTabs[index]))));
    };

    const BrowserContainer = deckyFrontendLib.findModuleChild((mod) => {
        if (typeof mod !== 'object')
            return undefined;
        for (let prop in mod) {
            if (mod[prop]?.toString().includes('displayURLBar') && mod[prop]?.toString().includes('BExternalTriggeredLoad()'))
                return mod[prop];
        }
    });

    const UsageWarningModal = ({ closeModal, onOk, onCancel }) => {
        return (window.SP_REACT.createElement(deckyFrontendLib.SimpleModal, { active: true },
            window.SP_REACT.createElement(deckyFrontendLib.ModalPosition, null,
                window.SP_REACT.createElement("div", { style: {
                        display: "flex",
                        flexDirection: "column",
                        flex: 1,
                        padding: '12px',
                    } },
                    window.SP_REACT.createElement(deckyFrontendLib.DialogSubHeader, { style: {
                            justifyContent: 'center',
                            display: 'flex',
                            margin: '0px 0px 30px 0px'
                        } }, "WARNING"),
                    window.SP_REACT.createElement(deckyFrontendLib.DialogBodyText, { style: { textAlign: 'justify' } }, "This plugin uses the Steam Deck's implementation of CEF which has known security vulnerabilities. Caution is advised when navigating external sites and it is not recommended to login to personal accounts. Use at your own risk.")),
                window.SP_REACT.createElement(deckyFrontendLib.DialogButtonSecondary, { onClick: () => {
                        closeModal();
                        onOk();
                    }, onCancelButton: () => {
                        closeModal();
                        onCancel();
                    } }, "I Understand"))));
    };

    var myFont = 'http://127.0.0.1:1337/plugins/Web Browser/assets/FORCED_SQUARE-ca77eef6.ttf';

    const tabBarPadding = 5;
    //tabHeight is defined by component and should not be changed
    const tabHeight = 34;
    const tabBarHeight = tabBarPadding * 2 + tabHeight;
    const BrowserStyling = () => {
        return window.SP_REACT.createElement("style", null, `
    @font-face {
        font-family: MyFont;
        src: url('${myFont}') format('truetype');
    }

    .tabAddonExit * {
        font-family: MyFont !important;
    }

    .${classes.tabbedBrowserContainer} .${deckyFrontendLib.gamepadTabbedPageClasses.Floating} .${deckyFrontendLib.gamepadTabbedPageClasses.TabContentsScroll} {
        clip: initial
    }
    .${classes.tabbedBrowserContainer} .${deckyFrontendLib.gamepadTabbedPageClasses.Right}>.${deckyFrontendLib.gamepadTabbedPageClasses.ContentTransition}.${deckyFrontendLib.gamepadTabbedPageClasses.Enter},
    .${classes.tabbedBrowserContainer} .${deckyFrontendLib.gamepadTabbedPageClasses.Right}>.${deckyFrontendLib.gamepadTabbedPageClasses.ContentTransition}.${deckyFrontendLib.gamepadTabbedPageClasses.EnterActive},
    .${classes.tabbedBrowserContainer} .${deckyFrontendLib.gamepadTabbedPageClasses.Right}>.${deckyFrontendLib.gamepadTabbedPageClasses.ContentTransition}.${deckyFrontendLib.gamepadTabbedPageClasses.Exit},
    .${classes.tabbedBrowserContainer} .${deckyFrontendLib.gamepadTabbedPageClasses.Right}>.${deckyFrontendLib.gamepadTabbedPageClasses.ContentTransition}.${deckyFrontendLib.gamepadTabbedPageClasses.ExitActive},
    .${classes.tabbedBrowserContainer} .${deckyFrontendLib.gamepadTabbedPageClasses.Left}>.${deckyFrontendLib.gamepadTabbedPageClasses.ContentTransition}.${deckyFrontendLib.gamepadTabbedPageClasses.Enter},
    .${classes.tabbedBrowserContainer} .${deckyFrontendLib.gamepadTabbedPageClasses.Left}>.${deckyFrontendLib.gamepadTabbedPageClasses.ContentTransition}.${deckyFrontendLib.gamepadTabbedPageClasses.EnterActive},
    .${classes.tabbedBrowserContainer} .${deckyFrontendLib.gamepadTabbedPageClasses.Left}>.${deckyFrontendLib.gamepadTabbedPageClasses.ContentTransition}.${deckyFrontendLib.gamepadTabbedPageClasses.Exit},
    .${classes.tabbedBrowserContainer} .${deckyFrontendLib.gamepadTabbedPageClasses.Left}>.${deckyFrontendLib.gamepadTabbedPageClasses.ContentTransition}.${deckyFrontendLib.gamepadTabbedPageClasses.ExitActive} {
        transform: initial;
    }
    .${classes.tabbedBrowserContainer} .${deckyFrontendLib.gamepadTabbedPageClasses.TabHeaderRowWrapper} {
        ${status.noTabBar ? 'display: none;' : ''}
        background: #060709;
    }
    .${classes.tabbedBrowserContainer} .${deckyFrontendLib.gamepadTabbedPageClasses.FixCenterAlignScroll} {
        padding: ${tabBarPadding}px 0px;
    }
    .${classes.tabbedBrowserContainer} .${deckyFrontendLib.gamepadTabbedPageClasses.TabContentsScroll} {
        bottom: var(--gamepadui-current-footer-height);
        padding: 0;
    }
    .${classes.tabbedBrowserContainer} .${browserClasses.ExternalBrowserContainer} {
        background: linear-gradient(to bottom, #060709 .5%, #1d2027 40%);
        top: ${status.noTabBar ? '0' : tabBarHeight}px;
    }
    .${classes.tabbedBrowserContainer} .${spinnerClasses.ContainerBackground} {
        background: transparent;
    }
    .${classes.tabbedBrowserContainer} .${spinnerClasses.LoadingStatus} {
        display: none;
    }
    .${classes.destructiveModal} button.${deckyFrontendLib.gamepadDialogClasses.Button}.DialogButton.gpfocus.Primary {
        background: #de3618;
        color: #fff
    }
    `);
    };
    function appendStyles(_window) {
        if (!_window)
            warnN('Styling', 'No SP Window');
        else {
            let style = _window.document.createElement('style');
            style.textContent = `
    .${classes.destructiveModal} button.${deckyFrontendLib.gamepadDialogClasses.Button}.DialogButton.gpfocus.Primary {
        background: #de3618;
        color: #fff
    }
    `;
            _window.document.head.appendChild(style);
        }
    }

    const BrowserMountAnimationContext = React.createContext({ done: false });
    let rerenderBrowser = () => { };
    const TabbedBrowser = ({ tabManager }) => {
        const [update, setUpdate] = React.useState(false);
        const [activeTab, setActiveTab] = React.useState(tabManager.tabHandlers.length > 0 ? tabManager.activeTab : null);
        const [mountAnimationFinished, setMountAnimationFinished] = React.useState(false);
        const activateTab = (id) => {
            tabManager.setActiveTabById(id);
            setActiveTab(id);
        };
        const onTitleChange = () => {
            setUpdate(state => !state);
        };
        const onCloseTab = () => {
            setActiveTab(tabManager.activeTab);
        };
        const onNewTab = () => {
            tabManager.tabHandlers[tabManager.tabHandlers.length - 1].registerOnTitleChange(onTitleChange);
            setActiveTab(tabManager.activeTab);
        };
        if (!status.running) {
            status.running = true;
            tabManager.createDefaultTabs();
        }
        React.useEffect(() => {
            rerenderBrowser = () => setUpdate(state => !state);
            if (!settingsManager.settings.seenWarning) {
                deckyFrontendLib.showModal(window.SP_REACT.createElement(UsageWarningModal, { closeModal: () => { }, onOk: () => { settingsManager.setSetting('seenWarning', true); }, onCancel: async () => {
                        deckyFrontendLib.Navigation.NavigateBack();
                        while (window.location.pathname === '/routes' + routePath) {
                            await deckyFrontendLib.sleep(100);
                        }
                        status.running = false;
                        tabManager.closeAllTabs();
                    } }));
            }
            (async () => {
                await tabManager.loadTabPromise;
                tabManager.setActiveBrowserHeader();
                setActiveTab(tabManager.activeTab);
                tabManager.registerOnNewTab(onNewTab);
                tabManager.registerOnCloseTab(onCloseTab);
                tabManager.tabHandlers.forEach(tabHandler => tabHandler.registerOnTitleChange(onTitleChange));
                setTimeout(() => {
                    setMountAnimationFinished(true);
                }, 800);
            })();
            return () => {
                if (tabManager.browserViewName === tabManager.headerStore.GetCurrentBrowserAndBackstack().browser.name) {
                    tabManager.headerStore.SetCurrentBrowserAndBackstack(null, false);
                }
            };
        }, []);
        tabManager.tabHandlers.forEach(tabHandler => {
            const footerActions = tabHandler.tab.content.props.focusableActionProps;
            if (!status.noTabBar) {
                if (footerActions?.onCancelButton) {
                    delete footerActions.onCancelButton;
                }
            }
            else {
                if (!footerActions?.onCancelButton)
                    footerActions.onCancelButton = () => deckyFrontendLib.Navigation.NavigateBack();
            }
        });
        return (window.SP_REACT.createElement("div", { style: { marginTop: '40px', height: 'calc(100% - 40px)', background: 'linear-gradient(to bottom, #060709 .5%, #1d2027 40%)' }, className: "tabbedBrowserContainer" },
            window.SP_REACT.createElement(BrowserStyling, null),
            !activeTab ? window.SP_REACT.createElement(deckyFrontendLib.SteamSpinner, null) :
                window.SP_REACT.createElement(BrowserMountAnimationContext.Provider, { value: { done: mountAnimationFinished } },
                    window.SP_REACT.createElement(deckyFrontendLib.Tabs, { activeTab: activeTab, onShowTab: activateTab, tabs: tabManager.tabHandlers.map(tabHandler => tabHandler.tab), autoFocusContents: true }))));
    };

    const BrowserTab = (props) => {
        const { browser, tabId, tabManager, getNavNode, clearNavNode, focusableActionProps } = props;
        const browserMountAnimation = React.useContext(BrowserMountAnimationContext);
        React.useEffect(() => {
            return () => clearNavNode();
        }, []);
        const element = (window.SP_REACT.createElement(deckyFrontendLib.Focusable, { noFocusRing: true, onGamepadFocus: (evt) => {
                // @ts-ignore
                if (evt.target?.classList[0] !== browserClasses.BrowserContainer) { //prevents from triggering twice
                    setTimeout(() => {
                        // @ts-ignore
                        evt.detail.focusedNode?.m_rgChildren[0]?.BTakeFocus(3);
                    }, browserMountAnimation.done ? 200 : 1100);
                }
            }, 
            //A button
            onOKButton: (evt) => {
                if (browser.m_gamepadBridge.GetGameInputSupportLevel().Value < 3) {
                    SteamClient.Input.ControllerKeyboardSetKeyState(88, true);
                    SteamClient.Input.ControllerKeyboardSetKeyState(88, false);
                }
            }, onGamepadDirection: (evt) => {
                if (browser.m_gamepadBridge.GetGameInputSupportLevel().Value < 3) {
                    switch (evt.detail.button) {
                        case 9:
                            //arrow up
                            SteamClient.Input.ControllerKeyboardSetKeyState(75, true);
                            SteamClient.Input.ControllerKeyboardSetKeyState(75, false);
                            break;
                        case 10:
                            //arrow down
                            SteamClient.Input.ControllerKeyboardSetKeyState(78, true);
                            SteamClient.Input.ControllerKeyboardSetKeyState(78, false);
                            break;
                        case 11:
                            //arrow left
                            SteamClient.Input.ControllerKeyboardSetKeyState(80, true);
                            SteamClient.Input.ControllerKeyboardSetKeyState(80, false);
                            break;
                        case 12:
                            //arrow right
                            SteamClient.Input.ControllerKeyboardSetKeyState(79, true);
                            SteamClient.Input.ControllerKeyboardSetKeyState(79, false);
                    }
                }
            }, ...focusableActionProps },
            window.SP_REACT.createElement(BrowserContainer, { browser: browser, className: browserClasses.ExternalBrowserContainer, visible: tabManager.activeTab === tabId && browserMountAnimation.done, hideForModals: true, external: true, displayURLBar: false, autoFocus: false })));
        getNavNode(element);
        return element;
    };

    const BrowserTabCloser = ({ close }) => {
        return window.SP_REACT.createElement("span", { onClick: () => close(), style: {
                height: 20,
                width: 20,
                backgroundColor: '#a0aaba88',
                borderRadius: '50%',
                display: 'inline-flex',
                justifyContent: 'center',
                alignItems: 'center',
                boxShadow: '#9598b2ba 0px 0px 3px 2px',
            }, className: "tabAddonExit" },
            window.SP_REACT.createElement("span", { style: { fontSize: '18px', transform: 'translate(1px, 1px)' } }, "X"));
    };

    const NewFavoriteModal = ({ tabManager, parentPath, closeModal }) => {
        const [editedUrl, setEditedUrl] = React.useState(tabManager.getActiveTabUrlRequested());
        const [favoriteName, setFavoriteName] = React.useState(tabManager.getActiveTabHandler().title);
        const [alreadyExists, setAlreadyExists] = React.useState(favoritesManager.doesExist([...parentPath, favoriteName], false));
        const updateFavoriteName = (newName) => {
            setFavoriteName(newName);
            setAlreadyExists(favoritesManager.doesExist([...parentPath, newName], false));
        };
        const dir = favoritesManager.pathToString(parentPath, '> ');
        // hacky way to select second text box on mount, focusOnMount doesnt work
        React.useEffect(() => {
            (async () => {
                const activeContext = deckyFrontendLib.getFocusNavController().m_ActiveContext;
                while (activeContext.m_ActiveFocusChange.to?.m_element.classList[0] !== deckyFrontendLib.gamepadDialogClasses.BasicTextInput) {
                    await deckyFrontendLib.sleep(5);
                }
                activeContext.m_ActiveFocusChange.to.m_Parent.m_rgChildren[0].m_Parent.m_Parent.m_rgChildren[1].BTakeFocus(3);
            })();
        }, []);
        return (window.SP_REACT.createElement(deckyFrontendLib.ConfirmModal, { className: alreadyExists ? classes.destructiveModal : '', strTitle: dir + favoriteName, strOKButtonText: alreadyExists ? 'Update ' + favoriteName : ('Save favorite' + (favoriteName ? ' as ' + favoriteName : '')), strCancelButtonText: 'Cancel', onOK: () => {
                const confirm = () => {
                    closeModal();
                    favoritesManager.addFavorite(favoriteName, editedUrl, parentPath);
                };
                if (favoritesManager.doesExist([...parentPath, favoriteName], false)) {
                    deckyFrontendLib.showModal(window.SP_REACT.createElement(ConfirmFavoriteOverwriteModal, { name: favoriteName, oldUrl: favoritesManager.getFavorite([...parentPath, favoriteName]), newUrl: editedUrl, onConfirm: confirm, closeModal: () => { } }));
                }
                else {
                    confirm();
                }
            }, onCancel: closeModal, bOKDisabled: !favoriteName || !editedUrl },
            window.SP_REACT.createElement(deckyFrontendLib.Marquee, { style: { position: 'relative', top: '-100px' }, play: true, center: true }, tabManager.getActiveTabUrlRequested()),
            window.SP_REACT.createElement(deckyFrontendLib.TextField, { description: 'Edit url', value: editedUrl, onChange: (evt) => {
                    setEditedUrl(evt.target.value);
                }, 
                //@ts-ignore
                onEnterKeyPress: () => { return 'VKClose'; } }),
            window.SP_REACT.createElement(deckyFrontendLib.TextField, { description: alreadyExists ? 'WARNING: ' + favoriteName + ' already exists in this location' : 'Enter a name for favorite', value: favoriteName, onChange: (evt) => {
                    updateFavoriteName(evt.target.value);
                }, 
                //@ts-ignore
                onEnterKeyPress: () => { return 'VKClose'; }, onFocus: (evt) => { evt.target.select(); } })));
    };

    const NewFavoriteFolderModal = ({ tabManager, parentPath, closeModal }) => {
        const [folderName, setFolderName] = React.useState('');
        const [alreadyExists, setAlreadyExists] = React.useState(false);
        const updateFolderName = (newName) => {
            setFolderName(newName);
            setAlreadyExists(favoritesManager.doesExist([...parentPath, newName], true));
        };
        const dir = favoritesManager.pathToString(parentPath, '> ');
        return (window.SP_REACT.createElement(deckyFrontendLib.ConfirmModal, { strTitle: dir + folderName, strOKButtonText: 'Make favorite' + (folderName ? ' in ' + folderName : ''), strCancelButtonText: 'Cancel', onOK: () => {
                closeModal();
                if (!favoritesManager.doesExist([...parentPath, folderName], true)) {
                    favoritesManager.addFolder(folderName, parentPath);
                }
                deckyFrontendLib.showModal(window.SP_REACT.createElement(NewFavoriteModal, { parentPath: [...parentPath, folderName], tabManager: tabManager, closeModal: () => { } }));
            }, onCancel: closeModal, bOKDisabled: !folderName },
            window.SP_REACT.createElement(deckyFrontendLib.Marquee, { style: { position: 'relative', top: '-100px' }, play: true, center: true }, tabManager.getActiveTabUrlRequested()),
            window.SP_REACT.createElement(deckyFrontendLib.TextField, { description: alreadyExists ? 'A folder with this name already exists in this location' : 'Enter a folder name', onChange: (evt) => {
                    updateFolderName(evt.target.value);
                }, 
                //@ts-ignore
                onEnterKeyPress: () => { return 'VKClose'; }, inlineControls: window.SP_REACT.createElement(deckyFrontendLib.DialogButton, { className: 'DialogInput_CopyAction', disabled: !folderName, focusable: !!folderName, style: { marginLeft: '25px' }, onClick: () => {
                        closeModal();
                        if (!favoritesManager.doesExist([...parentPath, folderName], true)) {
                            favoritesManager.addFolder(folderName, parentPath);
                        }
                        deckyFrontendLib.showModal(window.SP_REACT.createElement(NewFavoriteFolderModal, { parentPath: [...parentPath, folderName], tabManager: tabManager, closeModal: () => { } }));
                    } }, 'Make folder within') })));
    };

    const formatFavoriteListMenu = (children, value, path, tabManager, menu) => {
        path[0] = 'Favorites';
        const label = path[path.length - 1];
        const showDeleteItemModal = () => {
            menu.instance.Hide();
            const closeModal = () => { };
            deckyFrontendLib.showModal(window.SP_REACT.createElement(ConfirmFavoriteDeleteModal, { path: path, isFolder: !!children, url: value, closeModal: closeModal }));
        };
        if (children) {
            if (children.length === 0)
                children[0] = window.SP_REACT.createElement(deckyFrontendLib.MenuItem, { onMenuButton: () => { menu.instance.Hide(); } }, "Empty");
            const groupProps = { label: label };
            if (path.length !== 1)
                groupProps.onSecondaryActionDescription = 'Delete';
            groupProps.onMenuButton = () => {
                menu.instance.Hide();
            };
            groupProps.onSecondaryButton = () => { showDeleteItemModal(); };
            return (window.SP_REACT.createElement(deckyFrontendLib.MenuGroup, { ...groupProps }, ...children));
        }
        return (window.SP_REACT.createElement(deckyFrontendLib.MenuItem, { actionDescriptionMap: { [deckyFrontendLib.GamepadButton.SELECT]: 'View Url' }, onSecondaryActionDescription: 'Delete', onOptionsActionDescription: 'Open in New Tab', onOKActionDescription: 'Open in Current Tab', onClick: () => { tabManager.activeTabLoad(value); }, 
            //X button
            onSecondaryButton: () => { showDeleteItemModal(); }, 
            //Y button
            onOptionsButton: () => {
                menu.instance.Hide();
                setTimeout(() => tabManager.createTab(value), 50);
            }, 
            //start button
            onMenuButton: () => { menu.instance.Hide(); }, onButtonDown: (evt) => {
                switch (evt.detail.button) {
                    case deckyFrontendLib.GamepadButton.SELECT:
                        const modal = { instance: null };
                        modal.instance = deckyFrontendLib.showModal(window.SP_REACT.createElement(deckyFrontendLib.ModalRoot, null,
                            window.SP_REACT.createElement(deckyFrontendLib.Focusable, { onOKActionDescription: 'Open', onOptionsActionDescription: 'Open in New Tab', noFocusRing: true, onActivate: () => {
                                    modal.instance.Close();
                                    menu.instance.Hide();
                                    setTimeout(() => tabManager.activeTabLoad(value), 50);
                                }, onOptionsButton: (evt) => {
                                    modal.instance.Close();
                                    menu.instance.Hide();
                                    setTimeout(() => tabManager.createTab(value), 50);
                                } },
                                window.SP_REACT.createElement(deckyFrontendLib.Marquee, { play: true, center: true }, value))));
                        break;
                }
            } }, label.slice(0, -2)));
    };
    const formatAddToFavoritesMenu = (children, value, path, tabManager, menu) => {
        let propName = path[path.length - 1];
        const label = path.length === 1 ? 'Add to Favorites' : propName;
        path[0] = 'Favorites';
        //what to return if level is a parent
        if (children)
            return (
            //@ts-ignore
            window.SP_REACT.createElement(deckyFrontendLib.MenuGroup, { label: label, onMenuButton: () => { menu.instance.Hide(); } },
                window.SP_REACT.createElement(deckyFrontendLib.MenuItem
                //@ts-ignore
                , { 
                    //@ts-ignore
                    className: deckyFrontendLib.gamepadContextMenuClasses.Positive, onClick: () => {
                        deckyFrontendLib.showModal(window.SP_REACT.createElement(NewFavoriteModal, { parentPath: path, tabManager: tabManager, closeModal: () => { } }));
                    }, onMenuButton: () => { menu.instance.Hide(); } }, "Save"),
                window.SP_REACT.createElement(deckyFrontendLib.MenuItem
                //@ts-ignore
                , { 
                    //@ts-ignore
                    className: deckyFrontendLib.gamepadContextMenuClasses.Emphasis, onClick: () => {
                        deckyFrontendLib.showModal(window.SP_REACT.createElement(NewFavoriteFolderModal, { parentPath: path, tabManager: tabManager, closeModal: () => { } }));
                    }, onMenuButton: () => { menu.instance.Hide(); } }, "New Folder"),
                children.length > 0 ? window.SP_REACT.createElement("div", { className: deckyFrontendLib.gamepadContextMenuClasses.ContextMenuSeparator }) : null,
                ...children));
        //what to return if level is not a parent
        return null;
    };
    const BrowserContextMenu = ({ menu, tabManager }) => {
        if (!favoritesManager.loaded) {
            warnN('Context Menu', 'Favorites have not loaded');
        }
        if (!settingsManager.settingsLoaded) {
            warnN('Context Menu', 'Settings have not loaded');
        }
        const addToFavoritesMenuItem = favoritesManager.loaded ?
            favoritesManager.map(formatAddToFavoritesMenu, [tabManager, menu]) :
            window.SP_REACT.createElement(deckyFrontendLib.MenuGroup, { label: 'Add to Favorites', disabled: true });
        const favoritesMenuItem = favoritesManager.loaded ?
            favoritesManager.map(formatFavoriteListMenu, [tabManager, menu]) :
            window.SP_REACT.createElement(deckyFrontendLib.MenuGroup, { label: 'Favorites', disabled: true });
        const activeTabHasTarget = tabManager.getActiveTabHandler().hasTarget;
        return (
        //@ts-ignore
        window.SP_REACT.createElement(deckyFrontendLib.Menu, { onMenuButton: () => { menu.instance.Hide(); } },
            window.SP_REACT.createElement(deckyFrontendLib.MenuItem
            //@ts-ignore
            , { 
                //@ts-ignore
                className: deckyFrontendLib.gamepadContextMenuClasses.Emphasis, actionDescriptionMap: { [deckyFrontendLib.GamepadButton.SELECT]: 'View Home Page Url' }, onClick: () => { tabManager.activeTabLoadHome(); }, disabled: !settingsManager.settingsLoaded, onButtonDown: (evt) => {
                    switch (evt.detail.button) {
                        case deckyFrontendLib.GamepadButton.SELECT:
                            const modal = { instance: null };
                            modal.instance = deckyFrontendLib.showModal(window.SP_REACT.createElement(deckyFrontendLib.ModalRoot, null,
                                window.SP_REACT.createElement(deckyFrontendLib.Focusable, { noFocusRing: true, onActivate: () => { modal.instance.Close(); } },
                                    window.SP_REACT.createElement(deckyFrontendLib.Marquee, { play: true, center: true }, settingsManager.settings.homeUrl))));
                    }
                } }, "Home"),
            window.SP_REACT.createElement(deckyFrontendLib.MenuItem, { onClick: () => { tabManager.getActiveTabBrowserView().Reload(); } }, "Refresh Page"),
            window.SP_REACT.createElement("div", { className: deckyFrontendLib.gamepadContextMenuClasses.ContextMenuSeparator }),
            addToFavoritesMenuItem,
            favoritesMenuItem,
            window.SP_REACT.createElement("div", { className: deckyFrontendLib.gamepadContextMenuClasses.ContextMenuSeparator }),
            window.SP_REACT.createElement(deckyFrontendLib.MenuItem, { onClick: () => {
                    const closeModal = () => { };
                    deckyFrontendLib.showModal(window.SP_REACT.createElement(deckyFrontendLib.ConfirmModal, { className: classes.destructiveModal, strTitle: 'Are you sure you want to set your Home Page as', strCancelButtonText: 'Cancel', onOK: () => {
                            tabManager.saveActiveTabAsHomePage();
                        }, onCancel: closeModal, closeModal: closeModal },
                        window.SP_REACT.createElement("div", { style: { height: 15 } },
                            window.SP_REACT.createElement(deckyFrontendLib.Marquee, { play: true }, tabManager.getActiveTabUrlRequested()))));
                } }, "Set as Home Page"),
            window.SP_REACT.createElement(deckyFrontendLib.MenuItem, { disabled: !settingsManager.settingsLoaded, onClick: () => {
                    settingsManager.settings.defaultTabs.push(tabManager.getActiveTabUrlRequested());
                    settingsManager.saveSetting('defaultTabs');
                } }, "Add to Default Tabs"),
            window.SP_REACT.createElement("div", { className: deckyFrontendLib.gamepadContextMenuClasses.ContextMenuSeparator }),
            window.SP_REACT.createElement(deckyFrontendLib.MenuItem, { onClick: () => {
                    status.noTabBar = !status.noTabBar;
                    rerenderBrowser();
                } }, "Toggle Tab Bar"),
            window.SP_REACT.createElement(deckyFrontendLib.MenuItem, { onClick: activeTabHasTarget ? () => tabManager.inspectActiveTab() : () => tabManager.openCefInspectorTab(), onOptionsButton: activeTabHasTarget ? () => {
                    menu.instance.Hide();
                    tabManager.openCefInspectorTab();
                } : () => { }, onOKActionDescription: activeTabHasTarget ? 'Inspect Tab' : 'Open Inspector', onOptionsActionDescription: activeTabHasTarget ? 'Open Inspector' : '' }, "Inspect"),
            window.SP_REACT.createElement(deckyFrontendLib.MenuItem
            //@ts-ignore
            , { 
                //@ts-ignore
                className: deckyFrontendLib.gamepadContextMenuClasses.Destructive, onClick: async () => {
                    deckyFrontendLib.Navigation.NavigateBack();
                    while (window.location.pathname === '/routes' + routePath) {
                        await deckyFrontendLib.sleep(100);
                    }
                    status.running = false;
                    tabManager.closeAllTabs();
                } }, "Kill Browser")));
    };

    function SearchIcon(props) {
        return (window.SP_REACT.createElement("svg", { xmlns: "http://www.w3.org/2000/svg", viewBox: "0 0 36 36", fill: "none", ...props },
            window.SP_REACT.createElement("path", { d: 'M27.5 24C29.4972 21.1283 30.3471 17.6129 29.8823 14.146C29.4174 10.679 27.6716 7.5117 24.9884 5.26751C22.3052 3.02332 18.8792 1.86488 15.3846 2.02023C11.8901 2.17559 8.58036 3.63349 6.10692 6.10692C3.63349 8.58036 2.17559 11.8901 2.02023 15.3846C1.86488 18.8792 3.02332 22.3052 5.26751 24.9884C7.5117 27.6716 10.679 29.4174 14.146 29.8823C17.6129 30.3471 21.1283 29.4972 24 27.5L30.26 33.77L30.62 33.41L33.44 30.59L33.8 30.23L27.5 24ZM16 25C14.22 25 12.4799 24.4722 10.9999 23.4832C9.51983 22.4943 8.36628 21.0887 7.68509 19.4442C7.0039 17.7996 6.82567 15.99 7.17294 14.2442C7.52021 12.4984 8.37737 10.8947 9.63605 9.63605C10.8947 8.37737 12.4984 7.52021 14.2442 7.17294C15.99 6.82567 17.7996 7.0039 19.4442 7.68509C21.0887 8.36628 22.4943 9.51983 23.4832 10.9999C24.4722 12.4799 25 14.22 25 16C25 18.387 24.0518 20.6761 22.364 22.364C20.6761 24.0518 18.387 25 16 25Z', fill: 'currentColor' })));
    }

    const FocusableInput = (deckyFrontendLib.findModuleChild((mod) => {
        if (typeof mod !== 'object')
            return undefined;
        for (let prop in mod) {
            if (mod[prop]?.toString().includes('virtualKeyboardProps') && mod[prop]?.toString().includes('BIsElementValidForInput'))
                return mod[prop];
        }
    }))?.('input');

    const killBrowser = async (onClose) => {
        if (window.location.pathname === '/routes' + routePath) {
            deckyFrontendLib.Navigation.NavigateBack();
        }
        while (window.location.pathname === '/routes' + routePath) {
            await deckyFrontendLib.sleep(100);
        }
        status.running = false;
        tabManager.closeAllTabs();
        onClose?.();
    };
    function playUISound(path) {
        //@ts-ignore
        if (settingsStore?.m_ClientSettings?.enable_ui_sounds)
            GamepadUIAudio.AudioPlaybackManager.PlayAudioURL(path);
    }
    function addClasses$2(...strings) {
        return strings.filter(string => string).join(' ');
    }

    let searchBarNavFocusable;
    const SearchBarInput = ({ tabManager }) => {
        const url = "";
        const [value, setValue] = React.useState(url);
        const [isFocused, setIsFocused] = React.useState(false);
        const ref = React.useRef(null);
        React.useEffect(() => {
            const url = tabManager.getActiveTabUrlRequested();
            setValue(url);
        }, [isFocused]);
        const addBackgroundClass = React.useCallback((isFocused) => {
            return isFocused ? searchBarClasses.WhiteBackground : '';
        }, []);
        const submit = React.useCallback(() => {
            tabManager.browserRequest(value);
            tabManager.focusActiveBrowser();
        }, [value]);
        const inputElt = window.SP_REACT.createElement(FocusableInput, { ref: ref, className: addClasses$2(searchBarClasses.SearchBox, searchBarClasses.Visible, addBackgroundClass(isFocused)), type: 'search', placeholder: 'Enter URL or Search Term', strEnterKeyLabel: 'Go', value: value, focusable: true, noFocusRing: true, preferredFocus: false, onFocusWithin: setIsFocused, onFocus: () => setTimeout(() => ref.current?.select(), 50), onChange: (evt) => setValue(evt.target.value), onKeyDown: (evt) => { if (evt.key === 'Enter')
                submit(); }, onEnterKeyPress: () => (submit(), 'VKClose'), spellCheck: false });
        React.useEffect(() => {
            deckyFrontendLib.afterPatch(inputElt.type, 'render', (_, ret) => {
                searchBarNavFocusable = ret?.props?.value;
                return ret;
            }, { singleShot: true });
        }, []);
        return (window.SP_REACT.createElement(window.SP_REACT.Fragment, null,
            window.SP_REACT.createElement("style", null, `
                .${browserClasses.InputSupportLevel} {
                    display: none !important;
                }
                .${searchBarClasses.SearchBox} {
                    transition-timing-function: ease-in;
                }
                .${searchBarClasses.SearchBox}:focus-within {
                    color: #0e141b;
                    opacity: 1;
                    caret-color: #000;
                }
            `),
            window.SP_REACT.createElement("div", { className: addClasses$2(searchBarClasses.SearchFieldBackground, addBackgroundClass(isFocused)) }),
            window.SP_REACT.createElement(SearchIcon, { className: addClasses$2(searchBarClasses.SearchIconLeft, addBackgroundClass(isFocused)), width: '24px', height: '24px' }),
            inputElt,
            window.SP_REACT.createElement(SearchIcon, { className: addClasses$2(searchBarClasses.SearchIconRight, addBackgroundClass(isFocused)), width: '24px', height: '24px' })));
    };

    const FallbackSearchModal = ({ tabManager, closeModal }) => {
        const url = "";
        const [value, setValue] = React.useState(url);
        const submit = React.useCallback(() => {
            tabManager.browserRequest(value);
            tabManager.focusActiveBrowser();
            closeModal?.();
        }, [value]);
        return (window.SP_REACT.createElement(deckyFrontendLib.ModalRoot, { closeModal: closeModal },
            window.SP_REACT.createElement("div", { className: 'DialogInput_Wrapper' },
                window.SP_REACT.createElement(FocusableInput, { className: `DialogInput ${deckyFrontendLib.gamepadDialogClasses.BasicTextInput}`, noFocusRing: true, placeholder: 'Enter URL or Search Term', strEnterKeyLabel: 'Go', onChange: (evt) => setValue(evt.target.value), onKeyDown: (evt) => { if (evt.key === 'Enter')
                        submit(); }, onEnterKeyPress: () => (submit(), 'VKClose'), onMenuButton: submit, onMenuActionDescription: 'Go' }))));
    };

    const searchBarPatchState = { isPatched: false };
    let unpatchSearchBar = () => { };
    const patchSearchBar = async () => {
        let searchBarRootNode;
        let t = 0;
        while (!searchBarRootNode) {
            searchBarRootNode = deckyFrontendLib.findInReactTree(reactTree, node => node?.type?.toString().includes('SetUniversalSearchFocused') && node?.type?.toString().includes('GetForceHeaderAfterResume'));
            if (searchBarRootNode)
                continue;
            if (t >= 20000) {
                logN('Search Bar Patch', `Failed to find search bar root node after ${t / 1000} sec.`);
                searchBarPatchState.isPatched = false;
                return;
            }
            t += 200;
            await deckyFrontendLib.sleep(200);
        }
        const orig = searchBarRootNode.type;
        const searchBarRootWrapper = (props) => {
            const ret = orig(props);
            if (window.location.pathname === '/routes' + routePath) {
                const res = findLocation(ret, node => {
                    if (!node.type)
                        return false;
                    const fnString = node.type.toString?.();
                    return fnString?.includes('GamepadUI.Search.Root') && fnString?.includes('SearchBox');
                });
                if (res) {
                    const [parent, key] = res;
                    parent[key] = window.SP_REACT.createElement(SearchBarInput, { tabManager: tabManager });
                    searchBarPatchState.isPatched = true;
                }
                else {
                    logN("Search Bar Patch", 'Failed to find search bar element to patch.');
                    searchBarPatchState.isPatched = false;
                }
            }
            return ret;
        };
        searchBarRootNode.type = searchBarRootWrapper;
        if (searchBarRootNode.alternate) {
            searchBarRootNode.alternate.type = searchBarRootNode.type;
        }
        unpatchSearchBar = () => {
            searchBarRootNode.type = orig;
            searchBarRootNode.alternate.type = searchBarRootNode.type;
        };
    };
    const findLocation = (parent, filter) => {
        const walkable = ['props', 'children'];
        if (!parent || typeof parent !== 'object')
            return;
        if (Array.isArray(parent)) {
            for (let i = 0; i < parent.length; i++) {
                if (!parent[i])
                    continue;
                if (filter(parent[i]))
                    return [parent, i];
            }
            for (let i = 0; i < parent.length; i++) {
                if (!parent[i])
                    continue;
                const next = findLocation(parent[i], filter);
                if (next)
                    return next;
            }
        }
        else {
            for (const prop of walkable) {
                if (!parent[prop])
                    continue;
                if (filter(parent[prop]))
                    return [parent, prop];
            }
            for (const prop of walkable) {
                if (!parent[prop])
                    continue;
                const next = findLocation(parent[prop], filter);
                if (next)
                    return next;
            }
        }
    };

    class BrowserTabHandler {
        constructor(id, browser, tabManager, targetId) {
            //make sure this is an arrow function so 'this' keeps its reference as this class instance
            this.onSetTitle = (title) => {
                if (title !== this.title) {
                    this.title = title;
                    if (title.length > 30)
                        title = title.slice(0, 30);
                    this.tab.title = title;
                    if (this.onTitleChange)
                        this.onTitleChange();
                }
            };
            this.getNavNode = (browserTabElement) => {
                if (!this.navNode) {
                    deckyFrontendLib.afterPatch(browserTabElement.type, 'render', (_, ret) => {
                        this.navNode = ret.props.value;
                        return ret;
                    }, { singleShot: true });
                }
            };
            this.clearNavNode = () => {
                this.navNode = null;
            };
            browser.m_browserView.on('set-title', this.onSetTitle);
            this.title = 'data:text/html,<body><%2Fbody>';
            this.id = id;
            this.browser = browser;
            this.navNode = null;
            this.targetId = targetId;
            this.hasTarget = !!targetId;
            const outerTabActionProps = {
                //X button
                onSecondaryButton: () => {
                    browser.m_browserView.SetFocus(false);
                    tabManager.closeTab(id);
                },
                //Y button
                onOptionsButton: () => {
                    tabManager.createTab();
                },
                // start button
                onMenuButton: () => {
                    const shownContextMenu = { instance: null };
                    shownContextMenu.instance = deckyFrontendLib.showContextMenu(window.SP_REACT.createElement(BrowserContextMenu, { menu: shownContextMenu, tabManager: tabManager }));
                },
                onButtonDown: (evt) => {
                    switch (evt.detail.button) {
                        case deckyFrontendLib.GamepadButton.REAR_LEFT_LOWER:
                            //shift
                            SteamClient.Input.ControllerKeyboardSetKeyState(101, true);
                            //tab
                            SteamClient.Input.ControllerKeyboardSetKeyState(43, true);
                            SteamClient.Input.ControllerKeyboardSetKeyState(43, false);
                            SteamClient.Input.ControllerKeyboardSetKeyState(101, false);
                            break;
                        case deckyFrontendLib.GamepadButton.REAR_RIGHT_LOWER:
                            //tab
                            SteamClient.Input.ControllerKeyboardSetKeyState(43, true);
                            SteamClient.Input.ControllerKeyboardSetKeyState(43, false);
                            break;
                        case deckyFrontendLib.GamepadButton.TRIGGER_LEFT:
                        case deckyFrontendLib.GamepadButton.REAR_LEFT_UPPER:
                            //page back
                            if (browser.m_URLRequested !== browser.m_history.entries[1].url) {
                                browser.m_browserView.GoBack();
                            }
                            break;
                        case deckyFrontendLib.GamepadButton.TRIGGER_RIGHT:
                        case deckyFrontendLib.GamepadButton.REAR_RIGHT_UPPER:
                            //page forward
                            browser.m_browserView.GoForward();
                            break;
                        case deckyFrontendLib.GamepadButton.SELECT:
                            //focus searchbar or show search modal
                            if (searchBarPatchState.isPatched && searchBarNavFocusable && searchBarNavFocusable.BTakeFocus)
                                searchBarNavFocusable.BTakeFocus();
                            else
                                deckyFrontendLib.showModal(window.SP_REACT.createElement(FallbackSearchModal, { tabManager: tabManager }));
                            break;
                    }
                },
                onOKActionDescription: '',
                actionDescriptionMap: {
                    [deckyFrontendLib.GamepadButton.REAR_LEFT_UPPER]: 'Tab',
                    [deckyFrontendLib.GamepadButton.REAR_RIGHT_UPPER]: 'Shift Tab',
                    [deckyFrontendLib.GamepadButton.REAR_LEFT_LOWER]: 'Pg Back',
                    [deckyFrontendLib.GamepadButton.REAR_RIGHT_LOWER]: 'Pg Forward',
                    [deckyFrontendLib.GamepadButton.SELECT]: 'Search',
                },
                onCancelActionDescription: 'Back',
                onSecondaryActionDescription: 'Close Tab',
                onOptionsActionDescription: 'New Tab',
                onMenuActionDescription: 'Options'
            };
            this.tab = {
                id: id,
                title: '',
                content: window.SP_REACT.createElement(BrowserTab, { browser: this.browser, tabId: id, tabManager: tabManager, getNavNode: this.getNavNode, clearNavNode: this.clearNavNode, focusableActionProps: outerTabActionProps }),
                renderTabAddon: () => window.SP_REACT.createElement(BrowserTabCloser, { close: () => tabManager.closeTab(id) }),
                footer: outerTabActionProps
            };
        }
        closeBrowser() {
            setTimeout(() => this.browser.Destroy(), 200);
        }
        registerOnTitleChange(handler) {
            this.onTitleChange = handler;
        }
        loadUrl(url) {
            this.browser.LoadURL(url);
        }
        takeFocus() {
            this.navNode?.BTakeFocus(3);
        }
    }

    function getDefaultExportFromCjs (x) {
    	return x && x.__esModule && Object.prototype.hasOwnProperty.call(x, 'default') ? x['default'] : x;
    }

    var isURL$1 = {exports: {}};

    var assertString = {exports: {}};

    (function (module, exports) {

    Object.defineProperty(exports, "__esModule", {
      value: true
    });
    exports.default = assertString;

    function _typeof(obj) { "@babel/helpers - typeof"; if (typeof Symbol === "function" && typeof Symbol.iterator === "symbol") { _typeof = function _typeof(obj) { return typeof obj; }; } else { _typeof = function _typeof(obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }; } return _typeof(obj); }

    function assertString(input) {
      var isString = typeof input === 'string' || input instanceof String;

      if (!isString) {
        var invalidType = _typeof(input);

        if (input === null) invalidType = 'null';else if (invalidType === 'object') invalidType = input.constructor.name;
        throw new TypeError("Expected a string but received a ".concat(invalidType));
      }
    }

    module.exports = exports.default;
    module.exports.default = exports.default;
    }(assertString, assertString.exports));

    var isFQDN = {exports: {}};

    var merge = {exports: {}};

    (function (module, exports) {

    Object.defineProperty(exports, "__esModule", {
      value: true
    });
    exports.default = merge;

    function merge() {
      var obj = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
      var defaults = arguments.length > 1 ? arguments[1] : undefined;

      for (var key in defaults) {
        if (typeof obj[key] === 'undefined') {
          obj[key] = defaults[key];
        }
      }

      return obj;
    }

    module.exports = exports.default;
    module.exports.default = exports.default;
    }(merge, merge.exports));

    (function (module, exports) {

    Object.defineProperty(exports, "__esModule", {
      value: true
    });
    exports.default = isFQDN;

    var _assertString = _interopRequireDefault(assertString.exports);

    var _merge = _interopRequireDefault(merge.exports);

    function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

    var default_fqdn_options = {
      require_tld: true,
      allow_underscores: false,
      allow_trailing_dot: false,
      allow_numeric_tld: false,
      allow_wildcard: false,
      ignore_max_length: false
    };

    function isFQDN(str, options) {
      (0, _assertString.default)(str);
      options = (0, _merge.default)(options, default_fqdn_options);
      /* Remove the optional trailing dot before checking validity */

      if (options.allow_trailing_dot && str[str.length - 1] === '.') {
        str = str.substring(0, str.length - 1);
      }
      /* Remove the optional wildcard before checking validity */


      if (options.allow_wildcard === true && str.indexOf('*.') === 0) {
        str = str.substring(2);
      }

      var parts = str.split('.');
      var tld = parts[parts.length - 1];

      if (options.require_tld) {
        // disallow fqdns without tld
        if (parts.length < 2) {
          return false;
        }

        if (!options.allow_numeric_tld && !/^([a-z\u00A1-\u00A8\u00AA-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF]{2,}|xn[a-z0-9-]{2,})$/i.test(tld)) {
          return false;
        } // disallow spaces


        if (/\s/.test(tld)) {
          return false;
        }
      } // reject numeric TLDs


      if (!options.allow_numeric_tld && /^\d+$/.test(tld)) {
        return false;
      }

      return parts.every(function (part) {
        if (part.length > 63 && !options.ignore_max_length) {
          return false;
        }

        if (!/^[a-z_\u00a1-\uffff0-9-]+$/i.test(part)) {
          return false;
        } // disallow full-width chars


        if (/[\uff01-\uff5e]/.test(part)) {
          return false;
        } // disallow parts starting or ending with hyphen


        if (/^-|-$/.test(part)) {
          return false;
        }

        if (!options.allow_underscores && /_/.test(part)) {
          return false;
        }

        return true;
      });
    }

    module.exports = exports.default;
    module.exports.default = exports.default;
    }(isFQDN, isFQDN.exports));

    var isIP = {exports: {}};

    (function (module, exports) {

    Object.defineProperty(exports, "__esModule", {
      value: true
    });
    exports.default = isIP;

    var _assertString = _interopRequireDefault(assertString.exports);

    function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

    /**
    11.3.  Examples

       The following addresses

                 fe80::1234 (on the 1st link of the node)
                 ff02::5678 (on the 5th link of the node)
                 ff08::9abc (on the 10th organization of the node)

       would be represented as follows:

                 fe80::1234%1
                 ff02::5678%5
                 ff08::9abc%10

       (Here we assume a natural translation from a zone index to the
       <zone_id> part, where the Nth zone of any scope is translated into
       "N".)

       If we use interface names as <zone_id>, those addresses could also be
       represented as follows:

                fe80::1234%ne0
                ff02::5678%pvc1.3
                ff08::9abc%interface10

       where the interface "ne0" belongs to the 1st link, "pvc1.3" belongs
       to the 5th link, and "interface10" belongs to the 10th organization.
     * * */
    var IPv4SegmentFormat = '(?:[0-9]|[1-9][0-9]|1[0-9][0-9]|2[0-4][0-9]|25[0-5])';
    var IPv4AddressFormat = "(".concat(IPv4SegmentFormat, "[.]){3}").concat(IPv4SegmentFormat);
    var IPv4AddressRegExp = new RegExp("^".concat(IPv4AddressFormat, "$"));
    var IPv6SegmentFormat = '(?:[0-9a-fA-F]{1,4})';
    var IPv6AddressRegExp = new RegExp('^(' + "(?:".concat(IPv6SegmentFormat, ":){7}(?:").concat(IPv6SegmentFormat, "|:)|") + "(?:".concat(IPv6SegmentFormat, ":){6}(?:").concat(IPv4AddressFormat, "|:").concat(IPv6SegmentFormat, "|:)|") + "(?:".concat(IPv6SegmentFormat, ":){5}(?::").concat(IPv4AddressFormat, "|(:").concat(IPv6SegmentFormat, "){1,2}|:)|") + "(?:".concat(IPv6SegmentFormat, ":){4}(?:(:").concat(IPv6SegmentFormat, "){0,1}:").concat(IPv4AddressFormat, "|(:").concat(IPv6SegmentFormat, "){1,3}|:)|") + "(?:".concat(IPv6SegmentFormat, ":){3}(?:(:").concat(IPv6SegmentFormat, "){0,2}:").concat(IPv4AddressFormat, "|(:").concat(IPv6SegmentFormat, "){1,4}|:)|") + "(?:".concat(IPv6SegmentFormat, ":){2}(?:(:").concat(IPv6SegmentFormat, "){0,3}:").concat(IPv4AddressFormat, "|(:").concat(IPv6SegmentFormat, "){1,5}|:)|") + "(?:".concat(IPv6SegmentFormat, ":){1}(?:(:").concat(IPv6SegmentFormat, "){0,4}:").concat(IPv4AddressFormat, "|(:").concat(IPv6SegmentFormat, "){1,6}|:)|") + "(?::((?::".concat(IPv6SegmentFormat, "){0,5}:").concat(IPv4AddressFormat, "|(?::").concat(IPv6SegmentFormat, "){1,7}|:))") + ')(%[0-9a-zA-Z-.:]{1,})?$');

    function isIP(str) {
      var version = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : '';
      (0, _assertString.default)(str);
      version = String(version);

      if (!version) {
        return isIP(str, 4) || isIP(str, 6);
      }

      if (version === '4') {
        return IPv4AddressRegExp.test(str);
      }

      if (version === '6') {
        return IPv6AddressRegExp.test(str);
      }

      return false;
    }

    module.exports = exports.default;
    module.exports.default = exports.default;
    }(isIP, isIP.exports));

    (function (module, exports) {

    Object.defineProperty(exports, "__esModule", {
      value: true
    });
    exports.default = isURL;

    var _assertString = _interopRequireDefault(assertString.exports);

    var _isFQDN = _interopRequireDefault(isFQDN.exports);

    var _isIP = _interopRequireDefault(isIP.exports);

    var _merge = _interopRequireDefault(merge.exports);

    function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

    function _slicedToArray(arr, i) { return _arrayWithHoles(arr) || _iterableToArrayLimit(arr, i) || _unsupportedIterableToArray(arr, i) || _nonIterableRest(); }

    function _nonIterableRest() { throw new TypeError("Invalid attempt to destructure non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); }

    function _unsupportedIterableToArray(o, minLen) { if (!o) return; if (typeof o === "string") return _arrayLikeToArray(o, minLen); var n = Object.prototype.toString.call(o).slice(8, -1); if (n === "Object" && o.constructor) n = o.constructor.name; if (n === "Map" || n === "Set") return Array.from(o); if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen); }

    function _arrayLikeToArray(arr, len) { if (len == null || len > arr.length) len = arr.length; for (var i = 0, arr2 = new Array(len); i < len; i++) { arr2[i] = arr[i]; } return arr2; }

    function _iterableToArrayLimit(arr, i) { if (typeof Symbol === "undefined" || !(Symbol.iterator in Object(arr))) return; var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"] != null) _i["return"](); } finally { if (_d) throw _e; } } return _arr; }

    function _arrayWithHoles(arr) { if (Array.isArray(arr)) return arr; }

    /*
    options for isURL method

    require_protocol - if set as true isURL will return false if protocol is not present in the URL
    require_valid_protocol - isURL will check if the URL's protocol is present in the protocols option
    protocols - valid protocols can be modified with this option
    require_host - if set as false isURL will not check if host is present in the URL
    require_port - if set as true isURL will check if port is present in the URL
    allow_protocol_relative_urls - if set as true protocol relative URLs will be allowed
    validate_length - if set as false isURL will skip string length validation (IE maximum is 2083)

    */
    var default_url_options = {
      protocols: ['http', 'https', 'ftp'],
      require_tld: true,
      require_protocol: false,
      require_host: true,
      require_port: false,
      require_valid_protocol: true,
      allow_underscores: false,
      allow_trailing_dot: false,
      allow_protocol_relative_urls: false,
      allow_fragments: true,
      allow_query_components: true,
      validate_length: true
    };
    var wrapped_ipv6 = /^\[([^\]]+)\](?::([0-9]+))?$/;

    function isRegExp(obj) {
      return Object.prototype.toString.call(obj) === '[object RegExp]';
    }

    function checkHost(host, matches) {
      for (var i = 0; i < matches.length; i++) {
        var match = matches[i];

        if (host === match || isRegExp(match) && match.test(host)) {
          return true;
        }
      }

      return false;
    }

    function isURL(url, options) {
      (0, _assertString.default)(url);

      if (!url || /[\s<>]/.test(url)) {
        return false;
      }

      if (url.indexOf('mailto:') === 0) {
        return false;
      }

      options = (0, _merge.default)(options, default_url_options);

      if (options.validate_length && url.length >= 2083) {
        return false;
      }

      if (!options.allow_fragments && url.includes('#')) {
        return false;
      }

      if (!options.allow_query_components && (url.includes('?') || url.includes('&'))) {
        return false;
      }

      var protocol, auth, host, hostname, port, port_str, split, ipv6;
      split = url.split('#');
      url = split.shift();
      split = url.split('?');
      url = split.shift();
      split = url.split('://');

      if (split.length > 1) {
        protocol = split.shift().toLowerCase();

        if (options.require_valid_protocol && options.protocols.indexOf(protocol) === -1) {
          return false;
        }
      } else if (options.require_protocol) {
        return false;
      } else if (url.slice(0, 2) === '//') {
        if (!options.allow_protocol_relative_urls) {
          return false;
        }

        split[0] = url.slice(2);
      }

      url = split.join('://');

      if (url === '') {
        return false;
      }

      split = url.split('/');
      url = split.shift();

      if (url === '' && !options.require_host) {
        return true;
      }

      split = url.split('@');

      if (split.length > 1) {
        if (options.disallow_auth) {
          return false;
        }

        if (split[0] === '') {
          return false;
        }

        auth = split.shift();

        if (auth.indexOf(':') >= 0 && auth.split(':').length > 2) {
          return false;
        }

        var _auth$split = auth.split(':'),
            _auth$split2 = _slicedToArray(_auth$split, 2),
            user = _auth$split2[0],
            password = _auth$split2[1];

        if (user === '' && password === '') {
          return false;
        }
      }

      hostname = split.join('@');
      port_str = null;
      ipv6 = null;
      var ipv6_match = hostname.match(wrapped_ipv6);

      if (ipv6_match) {
        host = '';
        ipv6 = ipv6_match[1];
        port_str = ipv6_match[2] || null;
      } else {
        split = hostname.split(':');
        host = split.shift();

        if (split.length) {
          port_str = split.join(':');
        }
      }

      if (port_str !== null && port_str.length > 0) {
        port = parseInt(port_str, 10);

        if (!/^[0-9]+$/.test(port_str) || port <= 0 || port > 65535) {
          return false;
        }
      } else if (options.require_port) {
        return false;
      }

      if (options.host_whitelist) {
        return checkHost(host, options.host_whitelist);
      }

      if (host === '' && !options.require_host) {
        return true;
      }

      if (!(0, _isIP.default)(host) && !(0, _isFQDN.default)(host, options) && (!ipv6 || !(0, _isIP.default)(ipv6, 6))) {
        return false;
      }

      host = host || ipv6;

      if (options.host_blacklist && checkHost(host, options.host_blacklist)) {
        return false;
      }

      return true;
    }

    module.exports = exports.default;
    module.exports.default = exports.default;
    }(isURL$1, isURL$1.exports));

    var isURL = /*@__PURE__*/getDefaultExportFromCjs(isURL$1.exports);

    // Unique ID creation requires a high quality random # generator. In the browser we therefore
    // require the crypto API and do not support built-in fallback to lower quality random number
    // generators (like Math.random()).
    let getRandomValues;
    const rnds8 = new Uint8Array(16);
    function rng() {
      // lazy load so that environments that need to polyfill have a chance to do so
      if (!getRandomValues) {
        // getRandomValues needs to be invoked in a context where "this" is a Crypto implementation.
        getRandomValues = typeof crypto !== 'undefined' && crypto.getRandomValues && crypto.getRandomValues.bind(crypto);

        if (!getRandomValues) {
          throw new Error('crypto.getRandomValues() not supported. See https://github.com/uuidjs/uuid#getrandomvalues-not-supported');
        }
      }

      return getRandomValues(rnds8);
    }

    /**
     * Convert array of 16 byte values to UUID string format of the form:
     * XXXXXXXX-XXXX-XXXX-XXXX-XXXXXXXXXXXX
     */

    const byteToHex = [];

    for (let i = 0; i < 256; ++i) {
      byteToHex.push((i + 0x100).toString(16).slice(1));
    }

    function unsafeStringify(arr, offset = 0) {
      // Note: Be careful editing this code!  It's been tuned for performance
      // and works in ways you may not expect. See https://github.com/uuidjs/uuid/pull/434
      return (byteToHex[arr[offset + 0]] + byteToHex[arr[offset + 1]] + byteToHex[arr[offset + 2]] + byteToHex[arr[offset + 3]] + '-' + byteToHex[arr[offset + 4]] + byteToHex[arr[offset + 5]] + '-' + byteToHex[arr[offset + 6]] + byteToHex[arr[offset + 7]] + '-' + byteToHex[arr[offset + 8]] + byteToHex[arr[offset + 9]] + '-' + byteToHex[arr[offset + 10]] + byteToHex[arr[offset + 11]] + byteToHex[arr[offset + 12]] + byteToHex[arr[offset + 13]] + byteToHex[arr[offset + 14]] + byteToHex[arr[offset + 15]]).toLowerCase();
    }

    const randomUUID = typeof crypto !== 'undefined' && crypto.randomUUID && crypto.randomUUID.bind(crypto);
    var native = {
      randomUUID
    };

    function v4(options, buf, offset) {
      if (native.randomUUID && !buf && !options) {
        return native.randomUUID();
      }

      options = options || {};
      const rnds = options.random || (options.rng || rng)(); // Per 4.4, set bits for version and `clock_seq_hi_and_reserved`

      rnds[6] = rnds[6] & 0x0f | 0x40;
      rnds[8] = rnds[8] & 0x3f | 0x80; // Copy bytes to buffer, if provided

      if (buf) {
        offset = offset || 0;

        for (let i = 0; i < 16; ++i) {
          buf[offset + i] = rnds[i];
        }

        return buf;
      }

      return unsafeStringify(rnds);
    }

    class TabManager {
        constructor(defaultUrl, windowRouter) {
            this.createTab = async (Url) => {
                const id = v4();
                const url = (Url && Url !== 'home') ? Url : (settingsManager.settings.homeUrl || this.fallbackUrl);
                const browser = this.windowRouter.CreateBrowserView(this.browserViewName);
                browser.LoadURL(`data:text/plain,${id}`);
                const tabHandler = new BrowserTabHandler(id, browser, this);
                this.tabHandlers.push(tabHandler);
                this.setActiveTabById(id);
                await waitForUniqueUriLoad(browser, id).then(() => {
                    return backendService.serverApi.callPluginMethod('assign_target', { frontendId: id });
                }).then((res) => {
                    if (!res.success)
                        warnN('Tab Manager', 'Backend method "assign_target" returned this message> ');
                    else {
                        tabHandler.targetId = res.result;
                        tabHandler.hasTarget = true;
                    }
                    return res;
                }).catch(({ msg }) => {
                    warnN('Tab Manager', msg);
                    return { success: false, result: undefined };
                });
                browser.LoadURL(url);
                if (this.onNewTab)
                    this.onNewTab();
            };
            this.fallbackUrl = defaultUrl;
            this.browserViewName = 'TabbedWebBrowser';
            this.tabHandlers = [];
            this.windowRouter = windowRouter;
            this.headerStore = windowRouter.HeaderStore;
            this.activeTab = "";
            this.defaultSearchEngine = SearchEngine.GOOGLE;
        }
        closeTab(id) {
            const index = this.tabHandlers.findIndex(tabHandler => tabHandler.id === id);
            this.tabHandlers[index].closeBrowser();
            if (this.tabHandlers.length === 1)
                this.createTab();
            if (id === this.activeTab) {
                this.setActiveTabByIndex(index === 0 ? index + 1 : index - 1);
            }
            this.tabHandlers.splice(index, 1);
            if (this.onCloseTab)
                this.onCloseTab();
        }
        createDefaultTabs() {
            if (!settingsManager.settingsLoaded) {
                warnN('Tab Manager', 'Settings have not loaded when trying to create default tabs. Using fallback url to create tab instead.');
                this.createTab();
            }
            else {
                for (let i = 0; i < settingsManager.settings.defaultTabs.length; i++) {
                    const tab = settingsManager.settings.defaultTabs[i];
                    const tabPromise = this.createTab(tab);
                    if (i === settingsManager.settings.defaultTabs.length - 1) {
                        this.loadTabPromise = tabPromise;
                    }
                }
            }
        }
        setActiveBrowserHeaderByIndex(index) {
            this.headerStore.SetCurrentBrowserAndBackstack(this.tabHandlers[index].browser, true);
        }
        setActiveBrowserHeader() {
            const index = this.getActiveTabIndex();
            this.setActiveBrowserHeaderByIndex(index);
        }
        setActiveTabByIndex(index) {
            const id = this.tabHandlers[index].id;
            this.activeTab = id;
            this.setActiveBrowserHeaderByIndex(index);
        }
        setActiveTabById(id) {
            this.activeTab = id;
            this.setActiveBrowserHeader();
        }
        getTabIndexById(id) {
            return this.tabHandlers.findIndex(tabHandler => tabHandler.id === id);
        }
        getActiveTabIndex() {
            return this.tabHandlers.findIndex(tabHandler => tabHandler.id === this.activeTab);
        }
        getActiveTabHandler() {
            return this.tabHandlers[this.getActiveTabIndex()];
        }
        getTabIdByTargetId(targetId) {
            return this.tabHandlers.find(tabHandler => tabHandler.targetId === targetId)?.id;
        }
        getActiveTabBrowserView() {
            return this.getActiveTabHandler().browser.m_browserView;
        }
        getActiveTabUrlRequested() {
            return this.getActiveTabHandler().browser.m_URLRequested;
        }
        registerOnCloseTab(handler) {
            this.onCloseTab = handler;
        }
        registerOnNewTab(handler) {
            this.onNewTab = handler;
        }
        browserRequest(input) {
            const browserHandler = this.getActiveTabHandler();
            if (input !== browserHandler.browser.m_URLRequested) {
                if (isURL(input, { protocols: ['https', 'http', 'ftp', 'file', 'smb'] }) ||
                    isOtherProtocol(input, ['https', 'http', 'ftp', 'file', 'smb']) ||
                    isURL(localhostToIp(input))) {
                    browserHandler.loadUrl(convertLocalhostIpIfNeeded(input));
                }
                else {
                    browserHandler.loadUrl(getQueriedUrl(input, settingsManager.settings.searchEngine || this.defaultSearchEngine));
                }
            }
        }
        focusActiveBrowser() {
            this.getActiveTabHandler().takeFocus();
        }
        activeTabLoad(Url) {
            const url = (Url && Url !== 'home') ? Url : (settingsManager.settings.homeUrl || this.fallbackUrl);
            this.getActiveTabHandler().loadUrl(url);
        }
        activeTabLoadHome() {
            this.getActiveTabHandler().loadUrl(settingsManager.settings.homeUrl || this.fallbackUrl);
        }
        saveActiveTabAsHomePage() {
            settingsManager.setSetting('homeUrl', this.getActiveTabUrlRequested());
        }
        closeAllTabs() {
            for (let i = 0; i < this.tabHandlers.length; i++) {
                this.tabHandlers[i].closeBrowser();
            }
            this.tabHandlers = [];
        }
        openCefInspectorTab(targetId) {
            const url = targetId ? 'localhost:8080/devtools/inspector.html?ws=localhost:8080/devtools/page/' + targetId : 'localhost:8080';
            this.createTab(url);
        }
        inspectActiveTab() {
            const targetId = this.getActiveTabHandler().targetId;
            targetId && this.openCefInspectorTab(targetId);
        }
    }
    function isOtherProtocol(input, protocols) {
        for (let protocol of protocols) {
            if (input.startsWith(protocol + '://'))
                return true;
        }
        return false;
    }
    function localhostToIp(input) {
        if (input.includes('localhost'))
            return input.replace(/localhost/g, '127.0.0.1');
        return '';
    }
    function convertLocalhostIpIfNeeded(input) {
        let splitIp;
        if (input.includes(':') && (splitIp = input.split(':'))[0] === '127.0.0.1') {
            splitIp[0] = 'localhost:';
            input = splitIp.join('');
        }
        return input;
    }
    function getQueriedUrl(query, searchEngine) {
        let prefix;
        switch (searchEngine) {
            case SearchEngine.GOOGLE:
                prefix = 'https://www.google.com/search?q=';
                break;
            case SearchEngine.BING:
                prefix = 'https://www.bing.com/search?q=';
                break;
            case SearchEngine.YAHOO:
                prefix = 'https://search.yahoo.com/search?p=';
                break;
            case SearchEngine.CUSTOM:
                prefix = settingsManager.settings.customSearchUrl ?? 'https://www.google.com/search?q=';
        }
        return prefix + query;
    }
    function waitForUniqueUriLoad(browser, id) {
        let onFinish;
        browser.m_browserView.on('finished-request', (title) => {
            if (title === `data:text/plain,${id}`) {
                onFinish();
            }
        });
        return Promise.race([new Promise((resolve) => {
                onFinish = resolve;
            }),
            new Promise((resolve, reject) => {
                setTimeout(() => {
                    reject({ msg: 'Could not load unique URI within time limit' });
                }, 1000);
            })]);
    }
    const tabManager = new TabManager(defaultUrl, windowRouter);

    var DefaultContext = {
      color: undefined,
      size: undefined,
      className: undefined,
      style: undefined,
      attr: undefined
    };
    var IconContext = React__default["default"].createContext && React__default["default"].createContext(DefaultContext);

    var __assign = window && window.__assign || function () {
      __assign = Object.assign || function (t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
          s = arguments[i];
          for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p)) t[p] = s[p];
        }
        return t;
      };
      return __assign.apply(this, arguments);
    };
    var __rest = window && window.__rest || function (s, e) {
      var t = {};
      for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0) t[p] = s[p];
      if (s != null && typeof Object.getOwnPropertySymbols === "function") for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
        if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i])) t[p[i]] = s[p[i]];
      }
      return t;
    };
    function Tree2Element(tree) {
      return tree && tree.map(function (node, i) {
        return React__default["default"].createElement(node.tag, __assign({
          key: i
        }, node.attr), Tree2Element(node.child));
      });
    }
    function GenIcon(data) {
      // eslint-disable-next-line react/display-name
      return function (props) {
        return React__default["default"].createElement(IconBase, __assign({
          attr: __assign({}, data.attr)
        }, props), Tree2Element(data.child));
      };
    }
    function IconBase(props) {
      var elem = function (conf) {
        var attr = props.attr,
          size = props.size,
          title = props.title,
          svgProps = __rest(props, ["attr", "size", "title"]);
        var computedSize = size || conf.size || "1em";
        var className;
        if (conf.className) className = conf.className;
        if (props.className) className = (className ? className + " " : "") + props.className;
        return React__default["default"].createElement("svg", __assign({
          stroke: "currentColor",
          fill: "currentColor",
          strokeWidth: "0"
        }, conf.attr, attr, svgProps, {
          className: className,
          style: __assign(__assign({
            color: props.color || conf.color
          }, conf.style), props.style),
          height: computedSize,
          width: computedSize,
          xmlns: "http://www.w3.org/2000/svg"
        }), title && React__default["default"].createElement("title", null, title), props.children);
      };
      return IconContext !== undefined ? React__default["default"].createElement(IconContext.Consumer, null, function (conf) {
        return elem(conf);
      }) : elem(DefaultContext);
    }

    // THIS FILE IS AUTO GENERATED
    function BiChevronRight (props) {
      return GenIcon({"tag":"svg","attr":{"viewBox":"0 0 24 24"},"child":[{"tag":"path","attr":{"d":"M10.707 17.707 16.414 12l-5.707-5.707-1.414 1.414L13.586 12l-4.293 4.293z"}}]})(props);
    }

    /** Type of indicator to use when CustomButton is focused*/
    var CustomButtonFocusMode;
    (function (CustomButtonFocusMode) {
        CustomButtonFocusMode[CustomButtonFocusMode["highlight"] = 0] = "highlight";
        CustomButtonFocusMode[CustomButtonFocusMode["ring"] = 1] = "ring";
    })(CustomButtonFocusMode || (CustomButtonFocusMode = {}));
    /** CSS class names for CustomButton component */
    var CustomButtonClasses;
    (function (CustomButtonClasses) {
        CustomButtonClasses["buttonContainer"] = "custom-button-container";
        CustomButtonClasses["button"] = "custom-button";
    })(CustomButtonClasses || (CustomButtonClasses = {}));
    /** A button component with many customizable options */
    const CustomButton = ({ audioSFX, noAudio, disabled, focusable, transparent, focusMode, onFocus, onGamepadFocus, onBlur, onGamepadBlur, onClick, style, className, containerStyle, containerClassName, focusClassName, onOKActionDescription, children, ...focusableProps }) => {
        const [focused, setFocused] = React.useState(false);
        const focusStyle = focusMode ?? CustomButtonFocusMode.highlight;
        const audioPath = `/sounds/${audioSFX ?? 'deck_ui_default_activation.wav'}`;
        const onClicked = (e) => {
            if (!disabled) {
                !noAudio && playUISound(audioPath);
                onClick?.(e);
            }
        };
        return (window.SP_REACT.createElement(deckyFrontendLib.Focusable
        //@ts-ignore
        , { 
            //@ts-ignore
            onClick: onClicked, className: addClasses$2(CustomButtonClasses.buttonContainer, containerClassName), style: containerStyle, onActivate: focusable ?? true ? onClicked : undefined, onGamepadFocus: (e) => { setFocused(true); onFocus?.(e); onGamepadFocus?.(e); }, onGamepadBlur: (e) => { setFocused(false); onBlur?.(e); onGamepadBlur?.(e); }, noFocusRing: !(focusMode ?? false), onOKActionDescription: disabled ? '' : onOKActionDescription, ...focusableProps },
            window.SP_REACT.createElement(deckyFrontendLib.DialogButton, { className: addClasses$2(CustomButtonClasses.button, className, focusStyle === CustomButtonFocusMode.highlight && focused && 'gpfocus', focused && focusClassName), style: Object.assign(transparent && (focusStyle === CustomButtonFocusMode.ring || !focused) ? { background: 'transparent' } : {}, style ?? {}), focusable: false, disabled: disabled }, children)));
    };

    // THIS FILE IS AUTO GENERATED
    function BsThreeDots (props) {
      return GenIcon({"tag":"svg","attr":{"fill":"currentColor","viewBox":"0 0 16 16"},"child":[{"tag":"path","attr":{"d":"M3 9.5a1.5 1.5 0 1 1 0-3 1.5 1.5 0 0 1 0 3zm5 0a1.5 1.5 0 1 1 0-3 1.5 1.5 0 0 1 0 3zm5 0a1.5 1.5 0 1 1 0-3 1.5 1.5 0 0 1 0 3z"}}]})(props);
    }

    /** CSS class names for CustomDropdown component */
    var CustomDropdownClasses;
    (function (CustomDropdownClasses) {
        CustomDropdownClasses["topLevel"] = "custom-dropdown-container";
        CustomDropdownClasses["label"] = "custom-dropdown-label";
        CustomDropdownClasses["selectionChanged"] = "selection-changed";
    })(CustomDropdownClasses || (CustomDropdownClasses = {}));
    /** A dropdown component with many customizable options */
    const CustomDropdown = ({ rgOptions, selectedOption: selectedOptionData, style, labelStyle, labelChangedStyle, containerClassName, strDefaultLabel, labelCenter, menuLabel, noDropdownIcon, customDropdownIcon, focusMode, transparent, onChange, useCustomModal: CustomModal, onMenuOpened, ...buttonProps }) => {
        const icon = customDropdownIcon ?? (CustomModal ? window.SP_REACT.createElement(BsThreeDots, { style: { margin: 'auto' } }) : window.SP_REACT.createElement("svg", { style: { height: '1em', margin: 'auto' }, xmlns: "http://www.w3.org/2000/svg", viewBox: "0 0 36 36", fill: "none" },
            window.SP_REACT.createElement("path", { d: "M17.98 26.54L3.20996 11.77H32.75L17.98 26.54Z", fill: "currentColor" })));
        const [selected, setSelected] = React.useState(rgOptions?.find(option => option.data === selectedOptionData));
        const [changed, setChanged] = React.useState(false);
        React.useEffect(() => {
            if (changed) {
                setTimeout(() => setChanged(false), 15);
            }
        }, [changed]);
        React.useEffect(() => {
            if (selected?.data !== selectedOptionData) {
                setChanged(true);
                setSelected(rgOptions?.find(option => option.data === selectedOptionData));
            }
        }, [selectedOptionData, rgOptions?.length]);
        const onSelect = (option) => {
            setChanged(true);
            setSelected(option);
            onChange?.(option);
        };
        const showDefaultMenu = () => {
            deckyFrontendLib.showContextMenu(window.SP_REACT.createElement(deckyFrontendLib.Menu, { label: menuLabel ?? '' }, rgOptions?.map(option => window.SP_REACT.createElement(deckyFrontendLib.MenuItem, { selected: option === selected, onClick: () => onSelect(option) }, option.label))));
            onMenuOpened?.();
        };
        return (window.SP_REACT.createElement(CustomButton, { containerClassName: addClasses$1(CustomDropdownClasses.topLevel, containerClassName), style: { padding: '10px 16px', ...style }, noAudio: true, focusMode: focusMode, transparent: transparent, onClick: () => {
                CustomModal ? deckyFrontendLib.showModal(window.SP_REACT.createElement(CustomModal, { onSelectOption: (option) => onSelect(option), selectedOption: selected?.data, rgOptions: rgOptions })) : rgOptions && showDefaultMenu();
            }, ...buttonProps },
            window.SP_REACT.createElement("div", { style: { display: 'flex', overflow: 'hidden' } },
                window.SP_REACT.createElement("div", { style: { overflow: 'hidden', flex: 'auto' } },
                    window.SP_REACT.createElement("div", { style: Object.assign({ textAlign: labelCenter ? 'center' : 'left', minHeight: '20px' }, changed ? labelChangedStyle : labelStyle), className: addClasses$1(CustomDropdownClasses.label, changed && CustomDropdownClasses.selectionChanged) }, selected?.label ?? strDefaultLabel)),
                !noDropdownIcon && (window.SP_REACT.createElement("div", { style: { display: 'flex', marginLeft: '1ch', flex: 'none' } }, icon)))));
    };
    /** Utility function to join strings for CSS class names omitting invalid values */
    function addClasses$1(...strings) {
        return strings.filter(string => string).join(' ');
    }

    const defaultSFX = 'deck_ui_tab_transition_01.wav';
    const defaultInvalidSFX = 'deck_ui_bumper_end_02.wav';
    const altSFX = 'deck_ui_misc_01.wav';
    const altInvalidSFX = 'deck_ui_message_toast.wav';
    /** Mode for which elements should have transparency in EnhancedSelector component*/
    var EnhancedSelectorTransparencyMode;
    (function (EnhancedSelectorTransparencyMode) {
        /** No elements have transparency*/
        EnhancedSelectorTransparencyMode[EnhancedSelectorTransparencyMode["none"] = 0] = "none";
        /** Selection box has transparency, buttons don't*/
        EnhancedSelectorTransparencyMode[EnhancedSelectorTransparencyMode["selection"] = 1] = "selection";
        /** All elements have transparency */
        EnhancedSelectorTransparencyMode[EnhancedSelectorTransparencyMode["all"] = 2] = "all";
        /** Buttons have transparency, selection box doesn't */
        EnhancedSelectorTransparencyMode[EnhancedSelectorTransparencyMode["buttons"] = 3] = "buttons";
    })(EnhancedSelectorTransparencyMode || (EnhancedSelectorTransparencyMode = {}));
    /** Mode for when to use focus ring vs highlight when focusing an element in EnhancedSelector component */
    var EnhancedSelectorFocusRingMode;
    (function (EnhancedSelectorFocusRingMode) {
        /** Always use highlight and not ring */
        EnhancedSelectorFocusRingMode[EnhancedSelectorFocusRingMode["never"] = 0] = "never";
        /** Use ring for transparent elements and highlight otherwise */
        EnhancedSelectorFocusRingMode[EnhancedSelectorFocusRingMode["transparentOnly"] = 1] = "transparentOnly";
        /** Always use ring and not highlight */
        EnhancedSelectorFocusRingMode[EnhancedSelectorFocusRingMode["always"] = 2] = "always";
    })(EnhancedSelectorFocusRingMode || (EnhancedSelectorFocusRingMode = {}));
    /** CSS class names for EnhanceSelector component */
    var EnhancedSelectorClasses;
    (function (EnhancedSelectorClasses) {
        EnhancedSelectorClasses["topLevel"] = "enhanced-selector";
        EnhancedSelectorClasses["dirIcon"] = "direction-icon";
        EnhancedSelectorClasses["dirButton"] = "direction-button";
        EnhancedSelectorClasses["right"] = "direction-right";
        EnhancedSelectorClasses["left"] = "direction-left";
    })(EnhancedSelectorClasses || (EnhancedSelectorClasses = {}));
    /** A configurable component that allows to select from a list of options by cycling with buttons or from a dropdown menu. */
    const EnhancedSelector = ({ rgOptions, selectedOption: selectedOptionData, onChange, noWrapAround, showDropdownIcon, focusDropdown, transparencyMode, fullWidth, selectionBoxWidth, spacing, buttonWidth, focusRingMode, alternateSFX, sfxMain, sfxInvalid, animate, animationDuration, disabled, ...dropdownProps }) => {
        const noWrap = noWrapAround ?? false;
        const setWidth = selectionBoxWidth !== undefined;
        const transparency = transparencyMode ?? EnhancedSelectorTransparencyMode.none;
        const transparentButtons = transparency === EnhancedSelectorTransparencyMode.buttons || transparency === EnhancedSelectorTransparencyMode.all;
        const transparentSelectionBox = transparency === EnhancedSelectorTransparencyMode.selection || transparency === EnhancedSelectorTransparencyMode.all;
        const ringMode = focusRingMode ?? EnhancedSelectorFocusRingMode.never;
        const mainSfx = sfxMain ?? alternateSFX ? altSFX : defaultSFX;
        const invalidSfx = sfxInvalid ?? alternateSFX ? altInvalidSFX : defaultInvalidSFX;
        const getFocusRingMode = (transparent) => {
            switch (ringMode) {
                case EnhancedSelectorFocusRingMode.always:
                    return CustomButtonFocusMode.ring;
                case EnhancedSelectorFocusRingMode.never:
                    return CustomButtonFocusMode.highlight;
                case EnhancedSelectorFocusRingMode.transparentOnly:
                    return transparent ? CustomButtonFocusMode.ring : CustomButtonFocusMode.highlight;
            }
        };
        const selectionBoxFocusMode = getFocusRingMode(transparentSelectionBox);
        const buttonFocusMode = getFocusRingMode(transparentButtons);
        const incomingIndex = React.useMemo(() => {
            const index = rgOptions.findIndex(option => option.data === selectedOptionData);
            return index !== -1 ? index : 0;
        }, [selectedOptionData, rgOptions.length]);
        const [selectedIndex, setSelecetedIndex] = React.useState(incomingIndex);
        const [animateLabelStart, setAnimateLabelStart] = React.useState({});
        React.useEffect(() => {
            if (selectedIndex !== incomingIndex)
                setSelecetedIndex(incomingIndex);
        }, [selectedOptionData, rgOptions.length]);
        const getSFX = (dir) => (noWrap && ((selectedIndex === rgOptions.length - 1 && dir === 1) || (selectedIndex === 0 && dir === -1))) ? invalidSfx : mainSfx;
        const getNewIndex = (current, dir) => {
            const max = rgOptions.length;
            if (dir > 0) {
                let newIndex = (current + 1) % max;
                return newIndex === 0 && noWrap ? max - 1 : newIndex;
            }
            else {
                let newIndex = current - 1;
                return newIndex < 0 ? (!noWrap ? max - 1 : 0) : newIndex;
            }
        };
        const shiftIndex = (dir) => {
            const newIndex = getNewIndex(selectedIndex, dir);
            if (newIndex !== selectedIndex) {
                setSelecetedIndex(newIndex);
                animate && setAnimateLabelStart({ transform: `translate(${100 * dir}%)` });
                onChange?.(rgOptions[newIndex]);
            }
        };
        const onDropdownSelect = (option) => {
            const index = rgOptions.indexOf(option);
            setSelecetedIndex(index);
            animate && setAnimateLabelStart({});
            onChange?.(rgOptions[index]);
        };
        const buttonMargin = spacing ? spacing : '12px';
        const buttonStyle = {
            width: buttonWidth ? buttonWidth : '40px',
            minHeight: '40px',
            minWidth: 'initial',
            padding: 'initial'
        };
        const buttonIconStyle = {
            height: '1.5em',
            width: '1.5em',
            display: 'block',
            margin: 'auto'
        };
        return (window.SP_REACT.createElement(window.SP_REACT.Fragment, null,
            window.SP_REACT.createElement(deckyFrontendLib.Focusable, { className: `${EnhancedSelectorClasses.topLevel}`, style: { display: 'flex', justifyContent: 'center' } },
                window.SP_REACT.createElement(CustomButton, { audioSFX: getSFX(-1), onClick: () => shiftIndex(-1), className: addClasses(EnhancedSelectorClasses.dirButton, EnhancedSelectorClasses.left), containerStyle: { marginRight: buttonMargin }, transparent: transparentButtons, focusMode: buttonFocusMode, style: buttonStyle, disabled: disabled, focusable: !disabled },
                    window.SP_REACT.createElement(BiChevronRight, { style: Object.assign({ transform: 'rotate(180deg)' }, buttonIconStyle), className: EnhancedSelectorClasses.dirIcon })),
                window.SP_REACT.createElement(CustomDropdown, { rgOptions: rgOptions, selectedOption: rgOptions?.[selectedIndex]?.data, focusable: focusDropdown === true && !disabled, onChange: onDropdownSelect, focusMode: selectionBoxFocusMode, transparent: transparentSelectionBox, noDropdownIcon: !showDropdownIcon, containerStyle: fullWidth && !setWidth ? { width: '100%' } : {}, style: setWidth ? { width: selectionBoxWidth, minWidth: selectionBoxWidth } : { minWidth: '100px' }, labelStyle: animate ? { transition: `transform ${animationDuration ?? 300}ms ease-out` } : {}, labelChangedStyle: animateLabelStart, disabled: disabled, ...dropdownProps }),
                window.SP_REACT.createElement(CustomButton, { audioSFX: getSFX(1), onClick: () => shiftIndex(1), className: addClasses(EnhancedSelectorClasses.dirButton, EnhancedSelectorClasses.right), containerStyle: { marginLeft: buttonMargin }, transparent: transparentButtons, focusMode: buttonFocusMode, style: buttonStyle, disabled: disabled, focusable: !disabled },
                    window.SP_REACT.createElement(BiChevronRight, { style: buttonIconStyle, className: EnhancedSelectorClasses.dirIcon })))));
    };
    /** Utility function to join strings for CSS class names omitting invalid values */
    function addClasses(...strings) {
        return strings.filter(string => string).join(' ');
    }

    const CustomSearchUrlModal = ({ value, onSave, closeModal }) => {
        const [url, setUrl] = React.useState(value);
        return (window.SP_REACT.createElement(deckyFrontendLib.ConfirmModal, { onOK: () => onSave(url), closeModal: closeModal, strTitle: 'Custom URL', strDescription: (window.SP_REACT.createElement("div", null,
                window.SP_REACT.createElement("div", null, "This should include the query string with the query parameter at the end. The query will be appended to this."),
                window.SP_REACT.createElement("div", { style: { marginBottom: '10px' } }, "Ex: https://www.google.com/search?q="))) },
            window.SP_REACT.createElement(deckyFrontendLib.TextField, { value: url, onChange: e => setUrl(e.target.value) })));
    };

    const openUrl = (url, inNewtab) => {
        if (!status.running) {
            status.running = true;
            tabManager.createTab(url);
            deckyFrontendLib.Router.CloseSideMenus();
            deckyFrontendLib.Router.Navigate(routePath);
        }
        else {
            if (inNewtab) {
                tabManager.createTab(url);
            }
            else {
                tabManager.activeTabLoad(url);
            }
            deckyFrontendLib.Router.CloseSideMenus();
            if (window.location.pathname !== '/routes' + routePath) {
                deckyFrontendLib.Router.Navigate(routePath);
            }
        }
    };
    const QAMContent = ({}) => {
        const [isRunning, setIsRunning] = React.useState(status.running);
        const defaultTabItems = React.useMemo(() => {
            const items = [];
            for (let i = 0; i < settingsManager.settings.defaultTabs.length; i++) {
                const tab = settingsManager.settings.defaultTabs[i];
                const label = tab === 'home' ? 'Home Page' : tab;
                items.push({
                    description: (window.SP_REACT.createElement("div", { style: { wordBreak: 'break-all', marginTop: '-8px' } }, label)),
                    data: {
                        tab: tab
                    },
                    position: i
                });
            }
            return items;
        }, [settingsManager.settings.defaultTabs.length]);
        return (window.SP_REACT.createElement(window.SP_REACT.Fragment, null,
            !settingsManager.settingsLoaded ?
                (window.SP_REACT.createElement(deckyFrontendLib.PanelSection, { title: 'Loading Settings' },
                    window.SP_REACT.createElement(deckyFrontendLib.SteamSpinner, null))) :
                window.SP_REACT.createElement(window.SP_REACT.Fragment, null,
                    window.SP_REACT.createElement(deckyFrontendLib.PanelSection, { title: 'Home Page' },
                        window.SP_REACT.createElement(deckyFrontendLib.PanelSectionRow, null,
                            window.SP_REACT.createElement(deckyFrontendLib.Field, { focusable: true, description: window.SP_REACT.createElement("div", { style: { wordBreak: 'break-all', marginTop: '-8px' } }, settingsManager.settings.homeUrl || defaultUrl), onOptionsActionDescription: status.running ? 'Open in New Tab' : '', onOKActionDescription: status.running ? 'Open in Current Tab' : 'Open', onOKButton: () => openUrl(settingsManager.settings.homeUrl || defaultUrl), onOptionsButton: () => { openUrl(settingsManager.settings.homeUrl || defaultUrl, true); } }))),
                    window.SP_REACT.createElement(deckyFrontendLib.PanelSection, { title: "Default Tabs" },
                        window.SP_REACT.createElement(deckyFrontendLib.PanelSectionRow, { style: { marginLeft: '-16px', marginRight: '-16px' } },
                            window.SP_REACT.createElement(ReorderableList, { entries: defaultTabItems, onSave: (entries) => {
                                    settingsManager.setSetting('defaultTabs', entries.map((entry) => entry.data.tab));
                                }, onButtonPress: (evt, entry) => {
                                    const tab = entry.data.tab;
                                    switch (evt.detail.button) {
                                        case deckyFrontendLib.GamepadButton.SECONDARY:
                                            if (tab !== 'home') {
                                                const closeModal = () => { };
                                                deckyFrontendLib.showModal(window.SP_REACT.createElement(ConfirmDeleteDefaultTabModal, { index: entry.position, closeModal: closeModal }));
                                            }
                                            break;
                                        case deckyFrontendLib.GamepadButton.OK:
                                            openUrl(tab);
                                            break;
                                        case deckyFrontendLib.GamepadButton.OPTIONS:
                                            openUrl(tab, true);
                                    }
                                }, reorderButton: deckyFrontendLib.GamepadButton.START, saveOrderOnOk: true, fieldProps: {
                                    onSecondaryActionDescription: 'Delete',
                                    onOptionsActionDescription: status.running ? 'Open in New Tab' : '',
                                    onOKActionDescription: status.running ? 'Open in Current Tab' : 'Open'
                                } }))),
                    window.SP_REACT.createElement(deckyFrontendLib.PanelSection, { title: 'Search Engine' },
                        window.SP_REACT.createElement(deckyFrontendLib.PanelSectionRow, null,
                            window.SP_REACT.createElement(deckyFrontendLib.DropdownItem, { bottomSeparator: settingsManager.settings.searchEngine === SearchEngine.CUSTOM ? 'none' : 'standard', layout: 'below', rgOptions: [
                                    { label: 'Google', data: SearchEngine.GOOGLE },
                                    { label: 'Bing', data: SearchEngine.BING },
                                    { label: 'Yahoo', data: SearchEngine.YAHOO },
                                    { label: 'Custom', data: SearchEngine.CUSTOM }
                                ], selectedOption: settingsManager.settings.searchEngine ?? SearchEngine.GOOGLE, onChange: (option) => {
                                    settingsManager.setSetting('searchEngine', option.data);
                                } })),
                        settingsManager.settings.searchEngine === SearchEngine.CUSTOM && window.SP_REACT.createElement(window.SP_REACT.Fragment, null,
                            window.SP_REACT.createElement(deckyFrontendLib.PanelSectionRow, null,
                                window.SP_REACT.createElement(deckyFrontendLib.ButtonItem, { description: settingsManager.settings.customSearchUrl, onClick: () => deckyFrontendLib.showModal(window.SP_REACT.createElement(CustomSearchUrlModal, { value: settingsManager.settings.customSearchUrl ?? '', onSave: url => settingsManager.setSetting('customSearchUrl', url) })), layout: 'below' }, "Edit Custom Url")))),
                    window.SP_REACT.createElement(deckyFrontendLib.PanelSection, { title: 'Tab Bar' },
                        window.SP_REACT.createElement(deckyFrontendLib.PanelSectionRow, null,
                            window.SP_REACT.createElement(deckyFrontendLib.ToggleField, { label: window.SP_REACT.createElement("div", { style: { display: 'flex', alignItems: 'baseline' } },
                                    window.SP_REACT.createElement("div", null, "Start Hidden"),
                                    window.SP_REACT.createElement("div", { style: { fontSize: '.7em', lineHeight: '.7em', marginLeft: '1ch' } }, "requires reload")), checked: !!settingsManager.settings.noTabBar, onChange: checked => settingsManager.setSetting('noTabBar', checked) }))),
                    window.SP_REACT.createElement(deckyFrontendLib.PanelSection, { title: 'Menu Position' },
                        window.SP_REACT.createElement(deckyFrontendLib.PanelSectionRow, null,
                            window.SP_REACT.createElement(deckyFrontendLib.Field, { label: window.SP_REACT.createElement("div", { style: { width: '100%' } },
                                    window.SP_REACT.createElement(EnhancedSelector, { rgOptions: [
                                            { label: '1', data: 1 },
                                            { label: '2', data: 2 },
                                            { label: '3', data: 3 },
                                            { label: '4', data: 4 },
                                            { label: '5', data: 5 },
                                            { label: '6', data: 6 },
                                            { label: '7', data: 7 },
                                            { label: '8', data: 8 },
                                            { label: '9', data: 9 },
                                        ], onChange: option => {
                                            settingsManager.setSetting('menuPosition', option.data);
                                        }, selectedOption: settingsManager.settings.menuPosition, fullWidth: true })) })))),
            window.SP_REACT.createElement(deckyFrontendLib.PanelSection, { title: 'other' },
                window.SP_REACT.createElement(deckyFrontendLib.PanelSectionRow, null,
                    window.SP_REACT.createElement(deckyFrontendLib.ButtonItem, { layout: 'below', onClick: () => openUrl('localhost:8080', true) }, "Open Inspector")),
                window.SP_REACT.createElement(deckyFrontendLib.PanelSectionRow, null,
                    window.SP_REACT.createElement(deckyFrontendLib.ButtonItem, { disabled: !isRunning, layout: 'below', onClick: () => killBrowser(() => setIsRunning(false)) }, "Kill Browser")),
                window.SP_REACT.createElement(deckyFrontendLib.PanelSectionRow, null,
                    window.SP_REACT.createElement(deckyFrontendLib.ButtonItem, { layout: 'below', onClick: () => openUrl('https://github.com/jessebofill/DeckWebBrowser#readme', true) }, "View Readme")))));
    };

    const PluginIcon = ({ size, style }) => {
        return (window.SP_REACT.createElement("svg", { xmlns: "http://www.w3.org/2000/svg", width: size, height: size, style: style, viewBox: "0 0 50 50" },
            window.SP_REACT.createElement("path", { fill: 'currentColor', d: "m7.95,6.7c2.03.11,4.04.31,5.96.68.97-1.08,2.35-1.78,3.9-1.83.33-.9.69-1.81,1.09-2.73.33-.76.76-1.56,1.25-2.35-4.66.91-8.86,3.13-12.2,6.24Z" }),
            window.SP_REACT.createElement("path", { fill: 'currentColor', d: "m16.05,34.46c-1.36-5.44-1.96-11.74-.9-18.73-.29-.18-.56-.38-.81-.6-5.38,3.4-11.67,9.74-13.08,17.73.62,1.87,1.45,3.63,2.46,5.28,2.63.59,6.29.8,10.26.53.03-1.7.83-3.21,2.07-4.2Z" }),
            window.SP_REACT.createElement("path", { fill: 'currentColor', d: "m38.57,36.48c-.09,0-.17.01-.26.01-1.04,0-2.02-.3-2.85-.8-.28.21-.56.43-.86.64-2.84,2.02-6.34,3.51-10.06,4.55-.29.7-.73,1.32-1.26,1.83,1.36,2.9,2.8,5.32,3.97,7.17,4.82-.43,9.24-2.23,12.88-5-.29-2.73-.78-5.57-1.56-8.4Z" }),
            window.SP_REACT.createElement("path", { fill: 'currentColor', d: "m12.54,11.54c-.01-.12-.01-.23-.02-.35-2.51-.42-5.25-.56-7.96-.61C1.75,14.56.08,19.4,0,24.63c3.12-5.8,8.12-10.34,12.54-13.09Z" }),
            window.SP_REACT.createElement("path", { fill: 'currentColor', d: "m22.1,14.72c-.77.86-1.81,1.47-2.99,1.71-.92,6.25-.4,11.92.81,16.85,2.13.17,3.91,1.55,4.66,3.45,2.84-.89,5.5-2.09,7.7-3.66.26-.18.5-.37.75-.56-.14-.48-.22-1-.22-1.52,0-1.61.7-3.06,1.8-4.06-2.78-4.76-6.79-9.05-12.52-12.21Z" }),
            window.SP_REACT.createElement("path", { fill: 'currentColor', d: "m19.56,44.25s-.06,0-.09,0c-1.57,0-2.99-.66-3.99-1.72-1.63.15-3.24.23-4.79.23-1.19,0-2.33-.05-3.42-.13,3.99,4.01,9.33,6.67,15.28,7.25-.95-1.61-1.99-3.49-2.99-5.62Z" }),
            window.SP_REACT.createElement("path", { fill: 'currentColor', d: "m23.95,11.21c6.1,3.39,10.94,8.2,14.39,14.28.5,0,.98.08,1.44.2,2.9-4.05,4.29-8.2,4.82-11.69-7.63-4.16-14.92-4.12-20.65-2.79Z" }),
            window.SP_REACT.createElement("path", { fill: 'currentColor', d: "m48.23,16.3c-.8,3.74-2.43,7.83-5.26,11.78.53.85.84,1.84.84,2.91,0,1.5-.6,2.87-1.58,3.86.62,2.14,1.12,4.37,1.48,6.7,3.91-4.41,6.28-10.21,6.28-16.56,0-2.98-.52-5.84-1.48-8.5-.09-.07-.18-.13-.28-.2Z" }),
            window.SP_REACT.createElement("path", { fill: 'currentColor', d: "m22.57,4.4c-.36.83-.68,1.65-.98,2.47.23.19.44.4.63.63,6.15-1.59,14.03-1.96,22.35,1.96C40.07,3.78,33.15.11,25.36,0c-1.17,1.48-2.2,3.02-2.79,4.4Z" })));
    };

    const patchMenu = () => {
        const menuNode = deckyFrontendLib.findInReactTree(reactTree, (node) => node?.memoizedProps?.navID == 'MainNavMenuContainer');
        if (!menuNode || !menuNode.return?.type) {
            logN('Menu Patch', 'Failed to find main menu root node.');
            return () => { };
        }
        const orig = menuNode.return.type;
        let patchedInnerMenu;
        const menuWrapper = (props) => {
            const ret = orig(props);
            if (!ret?.props?.children?.props?.children?.[0]?.type) {
                logN('Menu Patch', 'The main menu element could not be found at the expected location. Valve may have changed it.');
                return ret;
            }
            if (patchedInnerMenu) {
                ret.props.children.props.children[0].type = patchedInnerMenu;
            }
            else {
                deckyFrontendLib.afterPatch(ret.props.children.props.children[0], 'type', (_, ret) => {
                    if (!ret?.props?.children || !Array.isArray(ret?.props?.children)) {
                        logN('Menu Patch', 'Could not find menu items to patch.');
                        return ret;
                    }
                    const itemIndexes = getMenuItemIndexes(ret.props.children);
                    const menuItemElement = deckyFrontendLib.findInReactTree(ret.props.children, (x) => x?.type?.toString()?.includes('exactRouteMatch:'));
                    const newItem = window.SP_REACT.createElement(MenuItemWrapper, { route: routePath, label: 'Browser', onFocus: menuItemElement.props.onFocus, MenuItemComponent: menuItemElement.type });
                    const browserPosition = settingsManager.settings.menuPosition;
                    if (browserPosition === 9)
                        ret.props.children.splice(itemIndexes[itemIndexes.length - 1] + 1, 0, newItem);
                    else
                        ret.props.children.splice(itemIndexes[browserPosition - 1], 0, newItem);
                    return ret;
                });
                patchedInnerMenu = ret.props.children.props.children[0].type;
            }
            return ret;
        };
        menuNode.return.type = menuWrapper;
        if (menuNode.return.alternate) {
            menuNode.return.alternate.type = menuNode.return.type;
        }
        return () => {
            menuNode.return.type = orig;
            menuNode.return.alternate.type = menuNode.return.type;
        };
    };
    function getMenuItemIndexes(items) {
        return items.flatMap((item, index) => (item && item.$$typeof && item.type !== 'div') ? index : []);
    }
    const MenuItemWrapper = ({ MenuItemComponent, ...props }) => {
        const [_, setState] = React.useState(false);
        return (window.SP_REACT.createElement(MenuItemComponent, { ...props, onSecondaryActionDescription: status.running ? 'Kill Browser' : '', onSecondaryButton: status.running ? () => killBrowser(() => setState((state => !state))) : undefined },
            window.SP_REACT.createElement(PluginIcon, { style: status.running ? { filter: 'drop-shadow(rgb(50, 255, 180) 0px 0px 8px)' } : {} })));
    };

    var index = deckyFrontendLib.definePlugin((serverApi) => {
        backendService.init(serverApi);
        settingsManager.init();
        favoritesManager.init();
        appendStyles(SP_Window);
        serverApi.routerHook.addRoute(routePath, () => { return window.SP_REACT.createElement(TabbedBrowser, { tabManager: tabManager }); });
        const unpatchMenu = patchMenu();
        patchSearchBar();
        const unregisterOnResume = SteamClient.System.RegisterForOnResumeFromSuspend(patchSearchBar).unregister;
        return {
            title: window.SP_REACT.createElement("div", { className: deckyFrontendLib.staticClasses.Title }, "Web Browser"),
            content: window.SP_REACT.createElement(QAMContent, null),
            icon: window.SP_REACT.createElement(PluginIcon, { size: '1em' }),
            onDismount() {
                serverApi.routerHook.removeRoute(routePath);
                unpatchMenu();
                unpatchSearchBar?.();
                unregisterOnResume();
            },
        };
    });

    return index;

})(DFL, SP_REACT);
